import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { AudioPlayback } from "../entities/audioPlayback.entity.js";
import { validateAudioPlayback } from "../validator/validateAudioPlayback.js";

// Add a new Audio Playback
export const addAudioPlayback = async (req, res) => {
  const { newAudioPlayback, status } = req.body;

  if (!newAudioPlayback) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingAudioPlayback = await AudioPlayback.findOne({
      $or: [{ newAudioPlayback }],
    });
    if (existingAudioPlayback) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.audioPlayback.audioPlaybackExits
      );
    } else {
      const { error } = validateAudioPlayback(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newLanguageData = new AudioPlayback({
          newAudioPlayback,
          status,
        });
        const savedAudioPlayback = await newLanguageData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.audioPlayback.success,
          {
            data: savedAudioPlayback,
          }
        );
      }
    }
  } catch (error) {
    logger.error(`Getting error while add audio playback : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Audio Playback
export const updateAudioPlayback = async (req, res) => {
  const { newAudioPlayback, status, _id } = req.body;
  const updatedAudioPlaybackData = { newAudioPlayback, status };
  try {
    const { error } = validateAudioPlayback(updatedAudioPlaybackData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedAudioPlayback = await AudioPlayback.findByIdAndUpdate(
        { _id: _id },
        { newAudioPlayback, status },
        { new: true }
      );
      if (!updatedAudioPlayback) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.audioPlayback.audioPlaybackNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.audioPlayback.audioPlaybackUpdated,
        {
          data: updatedAudioPlayback,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while update audio playback : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all Audio Playback
export const getAudioPlaybackes = async (req, res) => {
  try {
    const audioPlaybackes = await AudioPlayback.find();

    if (!audioPlaybackes.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      audioPlaybackes,
    });
  } catch (error) {
    logger.error(`Getting error while get audio playbackes : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Audio Playback
export const deleteAudioPlayback = async (req, res) => {
  const { id } = req.params;
  try {
    await AudioPlayback.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.audioPlayback.audioPlaybackDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete audio playback : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Audio Playback By Id
export const getAudioPlaybackById = async (req, res) => {
  try {
    const audioPlayback = await AudioPlayback.findOne({ _id: req.params.id });
    if (!audioPlayback) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.audioPlayback.audioPlaybackNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      audioPlayback,
    });
  } catch (error) {
    console.error("Error fetching user:", error);
    logger.error(`Getting error while  audio playback by id : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
