import bcrypt from "bcrypt";
import jwt from "jsonwebtoken";
import { StatusCode, USERTYPE } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import {
  getTokenUserInfo,
  sendResponse,
} from "../../../service/common.service.js";
import { generateOtp } from "../../../service/otpService.js";
import moment from "moment";
import { AdminUser, AppUser, User } from "../../user/entities/userSchema.js";
import { Role } from "../../role/entities/role.entity.js";
import { AuthTokenSchema } from "../entities/authTokenSchema.js";
import { logger } from "../../../service/logger.service.js";
import { sendEmail } from "../../../service/email.service.js";
import mongoose from "mongoose";

//*************sing-in********************//
export const signIn = async (req, res) => {
  const { email, password } = req.body;
  if (!email || !password) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const user = await User.findOne({ email });
    if (!user || !user.password) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.signIn.invalidCredentials
      );
    }
    const isPasswordValid = await bcrypt.compare(password, user.password);
    if (!isPasswordValid) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.signIn.invalidCredentials
      );
    }

    const token = jwt.sign(
      {
        _id: user._id,
        username: user.username,
        email: user.email,
        userType: user.userType,
        status: user.status,
        roleId: user.role,
      },
      process.env.JWT_SECRET,
      { expiresIn: "365d" }
    );

    let users;
    let permission;
    await AuthTokenSchema.create({
      userId: user?._id,
      token: token,
    });
    if (user?.status) {
      if (user?.userType == USERTYPE.ADMIN) {
        users = await AdminUser.findOne({ userId: user?._id });
        permission = await Role.findOne({ _id: user?.role });
        return sendResponse(
          res,
          StatusCode.OK,
          messages.signIn.success,
          {
            ...users._doc,

            permission,
          },
          token
        );
      } else {
        users = await AppUser.findOne({ userId: user?._id });

        if (users?.isVerify) {
          users.loginCount += 1;
          await users.save();

          return sendResponse(
            res,
            StatusCode.OK,
            messages.signIn.success,
            {
              ...users._doc,

              permission,
            },
            token
          );
        } else {
          const otp = generateOtp();
          user.otp = otp;
          await user.save();

          let emailHTML = `
          <div class="coupon" style="
            border: 0px;
            width: 95%;
            border-radius: 0px;
            margin: 0 auto;
            max-width: 600px;
            font-family: Arial;
          ">
          <div style="
              padding: 23px 16px;
              background-color: #c4b4fc;
              text-align: center;
              border-radius: 0 0;
            ">
            <a href="#">
              <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                style="padding: 10px 0px;max-width: 200px; height: auto;" />
            </a>
          </div>
          <div style="background-color: white; padding: 0 16px">
            <p style="
                font-size: 14px;
                font-family: Arial, sans-serif;
                color: #2b303a;
                margin: 25px 0 0;
                text-transform: capitalize;
              ">
              Dear <b>${user.username}</b>
            </p>
          </div>
          <div style="font-family: sans-serif; padding: 10px 16px 50px">
            <div class="" style="
                font-size: 12px;
                font-family: Arial, sans-serif;
                mso-line-height-alt: 18px;
                color: #555555;
                line-height: 1.5;
              ">
              <p style="
                  margin: 0;
                  font-size: 14px;
                  text-align: left;
                  mso-line-height-alt: 22.5px;
                "></p>
              <p>Please use this code to verify your email.</p>
              <p style="
                  font-size: 16px;">
                <b>OTP : </b> ${otp}
              </p>
              <p>
                <span>Best Regards,</span><br /><span><b>Summary AI Team</b></span>
              </p>
            </div>
          </div>

          <div style="background-color: #c4b4fc">
            <div style="
                padding: 2px 16px;
                text-align: center;
                border-radius: 0 0;
                border: 0;
              ">
              <a href="#" style="border: 0">
                <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                  style="padding: 10px 0px;max-width: 200px; height: auto;" />
              </a>
            </div>
            <div align="center" class="alignment" style="padding: 2px 16px; text-align: center; border-radius: 0 0">
              <div style="padding: 2px 16px; text-align: center; border-radius: 0 0">
                <div class="" style="
                    font-size: 14px;
                    font-family: Arial, sans-serif;
                    mso-line-height-alt: 14.399999999999999px;
                    color: #555555;
                    line-height: 1.2;
                  ">
                  <p style="
                      margin: 0;
                      font-size: 14px;
                      text-align: center;
                      mso-line-height-alt: 16.8px;
                      padding: 0 0 18px 0;
                    ">
                    <span style="color: #000; font-size: 14px">Copyright © 2024 Summary AI Services - All rights
                      reserved.</span>
                  </p>
                </div>
              </div>
            </div>
          </div>
        </div>`;

          const emailSent = await sendEmail({
            from: process.env.SENDER_EMAIL,
            to: user.email,
            subject: "Summary AI Verification OTP",
            html: emailHTML,
          });

          if (!emailSent) {
            return sendResponse(
              res,
              StatusCode.INTERNAL_SERVER_ERROR,
              "Error sending verification email"
            );
          }

          return sendResponse(res, StatusCode.OK, messages.signIn.otpSent, {
            otp,
            ...users._doc,
          });
        }
      }
    } else {
      return sendResponse(
        res,
        StatusCode.UNAUTHORIZED,
        messages.unauthorizedUser
      );
    }
  } catch (error) {
    console.error("Error during sign-in:", error);
    logger.error(`Getting error while sign-In : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

//*************verify-OTP********************//
export const verifyOTP = async (req, res) => {
  const { email, otp } = req.body;

  if (!email || !otp) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const user = await User.findOne({ email });
    if (!user || user.otp !== otp) {
      return sendResponse(
        res,
        StatusCode.UNAUTHORIZED,
        messages.verifyOTP.invalidOTP
      );
    }

    // Clear OTP from user document
    user.otp = null;
    await user.save();

    const token = jwt.sign(
      {
        _id: user._id,
        username: user.username,
        email: user.email,
        userType: user.userType,
        status: user.status,
        roleId: user.role,
      },
      process.env.JWT_SECRET,
      { expiresIn: "365d" }
    );
    await AuthTokenSchema.create({
      userId: user?._doc._id,
      token: token,
    });

    if (user._doc.userType == USERTYPE.ADMIN) {
      return sendResponse(
        res,
        StatusCode.OK,
        messages.verifyOTP.success,
        {
          ...user._doc,
        },
        token
      );
    } else {
      const appUser = await AppUser.findOne({ userId: user._doc._id });
      appUser.isVerify = true;
      await appUser.save();

      return sendResponse(
        res,
        StatusCode.OK,
        messages.verifyOTP.success,
        {
          ...appUser._doc,
        },
        token
      );
    }
  } catch (error) {
    console.error("Error during OTP verification:", error);
    logger.error(`Getting error while OTP verification : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

//*************resend-OTP********************//
export const resendOTP = async (req, res) => {
  const { email } = req.body;
  const currentTime = moment();

  if (!email) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const user = await User.findOne({ email });
    if (!user) {
      return sendResponse(res, StatusCode.UNAUTHORIZED, messages.noUsersFound);
    }

    if (user) {
      if (user.otpSentTime) {
        const dateTimeDiff = currentTime.diff(
          moment(user.otpSentTime),
          "minutes"
        );
        if (dateTimeDiff < 2) {
          return sendResponse(
            res,
            StatusCode.TOO_MANY_REQUESTS,
            messages.resendOTP.tooSoon
          );
        }
      }
    }
    // Generate new OTP
    const otp = generateOtp();
    user.otp = otp;
    await user.save();

    let emailHTML = `
          <div class="coupon" style="
            border: 0px;
            width: 95%;
            border-radius: 0px;
            margin: 0 auto;
            max-width: 600px;
            font-family: Arial;
          ">
          <div style="
              padding: 23px 16px;
              background-color: #c4b4fc;
              text-align: center;
              border-radius: 0 0;
            ">
            <a href="#">
              <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                style="padding: 10px 0px;max-width: 200px; height: auto;" />
            </a>
          </div>
          <div style="background-color: white; padding: 0 16px">
            <p style="
                font-size: 14px;
                font-family: Arial, sans-serif;
                color: #2b303a;
                margin: 25px 0 0;
                text-transform: capitalize;
              ">
              Dear <b>${user.username}</b>
            </p>
          </div>
          <div style="font-family: sans-serif; padding: 10px 16px 50px">
            <div class="" style="
                font-size: 12px;
                font-family: Arial, sans-serif;
                mso-line-height-alt: 18px;
                color: #555555;
                line-height: 1.5;
              ">
              <p style="
                  margin: 0;
                  font-size: 14px;
                  text-align: left;
                  mso-line-height-alt: 22.5px;
                "></p>
              <p>Please use this code to verify your email.</p>
              <p style="
                  font-size: 16px;">
                <b>OTP : </b> ${otp}
              </p>
              <p>
                <span>Best Regards,</span><br /><span><b>Summary AI Team</b></span>
              </p>
            </div>
          </div>

          <div style="background-color: #c4b4fc">
            <div style="
                padding: 2px 16px;
                text-align: center;
                border-radius: 0 0;
                border: 0;
              ">
              <a href="#" style="border: 0">
                <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                  style="padding: 10px 0px;max-width: 200px; height: auto;" />
              </a>
            </div>
            <div align="center" class="alignment" style="padding: 2px 16px; text-align: center; border-radius: 0 0">
              <div style="padding: 2px 16px; text-align: center; border-radius: 0 0">
                <div class="" style="
                    font-size: 14px;
                    font-family: Arial, sans-serif;
                    mso-line-height-alt: 14.399999999999999px;
                    color: #555555;
                    line-height: 1.2;
                  ">
                  <p style="
                      margin: 0;
                      font-size: 14px;
                      text-align: center;
                      mso-line-height-alt: 16.8px;
                      padding: 0 0 18px 0;
                    ">
                    <span style="color: #000; font-size: 14px">Copyright © 2024 Summary AI Services - All rights
                      reserved.</span>
                  </p>
                </div>
              </div>
            </div>
          </div>
        </div>`;

    const emailSent = await sendEmail({
      from: process.env.SENDER_EMAIL,
      to: user.email,
      subject: "Summary AI Verification OTP",
      html: emailHTML,
    });

    if (!emailSent) {
      return sendResponse(
        res,
        StatusCode.INTERNAL_SERVER_ERROR,
        "Error sending verification email"
      );
    }

    return sendResponse(res, StatusCode.OK, messages.signIn.otpSent, {
      otp,
      email,
    });
  } catch (error) {
    console.error("Error during OTP resend:", error);
    logger.error(`Getting error while OTP resend : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const resetPassword = async (req, res) => {
  const { password, resetToken } = req.body;

  try {
    const decoded = jwt.verify(resetToken, process.env.JWT_SECRET);

    if (decoded) {
      const user = await User.findOne({
        _id: decoded._id,
      });
      if (!user) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          messages.signIn.invalidCredentials
        );
      }
      const hashedPassword = await bcrypt.hash(password, 10);
      // Update user's password with the new hashed password
      user.password = hashedPassword;
      await user.save();

      return sendResponse(res, StatusCode.OK, messages.resetPassword.success);
    } else {
      return sendResponse(
        res,
        StatusCode.UNAUTHORIZED,
        messages.resetPassword.invalidToken
      );
    }
  } catch (error) {
    logger.error(`Getting error while reset password : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const changePassword = async (req, res) => {
  const { userId, currentPassword, newPassword } = req.body;

  if (!userId || !currentPassword || !newPassword) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }

  try {
    const user = await User.findById(userId);
    if (!user) {
      return sendResponse(res, StatusCode.NOT_FOUND, messages.noUsersFound);
    }

    const isPasswordMatch = await bcrypt.compare(
      currentPassword,
      user.password
    );
    if (!isPasswordMatch) {
      return sendResponse(
        res,
        StatusCode.UNAUTHORIZED,
        messages.incorrectPassword
      );
    }

    const hashedNewPassword = await bcrypt.hash(newPassword, 10);

    user.password = hashedNewPassword;
    await user.save();

    return sendResponse(res, StatusCode.OK, messages.changePassword.success);
  } catch (error) {
    logger.error(`Error changing password: ${error}`);
    console.error("Error changing password:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
export const logOutDevice = async (req, res) => {
  try {
    const userInfo = getTokenUserInfo(req);
    const userId = new mongoose.Types.ObjectId(userInfo._id);
    const user = await AuthTokenSchema.findOne({
      userId,
    });
    if (user && user.token) {
      await AuthTokenSchema.findOneAndDelete({
        userId: userId,
      });
      return sendResponse(res, StatusCode.OK, messages.logOutSuccess);
    } else {
      return sendResponse(res, StatusCode.BADREQUEST, messages.tokenNotFound);
    }
  } catch (error) {
    console.error("Error logging out from all devices:", error);
    return res.status(500).json({
      message: "An error occurred while logging out from all devices.",
    });
  }
};

export const googleAuth = async (req, res) => {
  try {
    const { username, email } = req.body;

    // Check if the email is provided
    if (!email) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.allFieldsRequired
      );
    }

    // Check if the user already exists
    const existingUser = await User.findOne({ email });

    let appUser;

    let userId;
    if (existingUser) {
      // Existing user found, update appUser details
      appUser = await AppUser.findOne({ userId: existingUser._id });
      appUser.isVerify = true;
      appUser.loginCount += 1;

      userId = existingUser._id;
    } else {
      // Create new user and corresponding appUser
      const newUser = new User({ username, email, status: true });
      const savedUser = await newUser.save();
      userId = savedUser._id;

      appUser = new AppUser({
        userId: savedUser._id,
        username: savedUser.username,
        email: savedUser.email,
        userType: savedUser.userType,
        status: savedUser.status,
        isVerify: true,
        loginCount: 1,
      });
    }

    // Save the appUser
    await appUser.save();

    // Generate JWT token
    const token = jwt.sign(
      {
        _id: userId,
        username: appUser.username,
        email: appUser.email,
        userType: appUser.userType,
        status: appUser.status,
        roleId: appUser.role,
      },
      process.env.JWT_SECRET,
      { expiresIn: "365d" }
    );

    // Store the token in AuthTokenSchema

    console.log("userId", userId);

    await AuthTokenSchema.create({ userId: userId, token });

    // Send the response
    return sendResponse(
      res,
      StatusCode.OK,
      messages.signIn.success,
      appUser,
      token
    );
  } catch (error) {
    console.error("Error during Google Authentication:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// export const googleAuth = async (req, res) => {
//   try {
//     const { username, email } = req.body;
//     if (!email) {
//       return sendResponse(
//         res,
//         StatusCode.BADREQUEST,
//         messages.allFieldsRequired
//       );
//     }

//     const query = { $or: [{ email }] };
//     const existingUser = await User.findOne(query);

//     if (existingUser) {
//       const appUser = await AppUser.findOne({ userId: existingUser._doc._id });
//       appUser.isVerify = true;
//       appUser.loginCount += 1;
//       await appUser.save();
//       const token = jwt.sign(
//         {
//           _id: appUser._doc._id,
//           username: appUser._doc.username,
//           email: appUser._doc.email,
//           userType: appUser._doc.userType,
//           status: appUser._doc.status,
//           roleId: appUser._doc.role,
//         },
//         process.env.JWT_SECRET,
//         { expiresIn: "365d" }
//       );

//       await AuthTokenSchema.create({
//         userId: appUser?._doc?.userId,
//         token: token,
//       });

//       return sendResponse(
//         res,
//         StatusCode.OK,
//         messages.signIn.success,
//         {
//           ...appUser._doc,
//         },
//         token
//       );
//     } else {
//       const newUser = new User({
//         username,
//         email,
//         status: true,
//       });

//       const savedUser = await newUser.save();

//       const appUser = new AppUser({
//         userId: savedUser._id,
//         username: savedUser.username,
//         email: savedUser.email,
//         userType: savedUser.userType,
//         status: savedUser.status,
//         isVerify: true,
//       });
//       await appUser.save();

//       if (appUser?.isVerify) {
//         appUser.loginCount += 1;
//         await appUser.save();

//         const token = jwt.sign(
//           {
//             _id: appUser._doc._id,
//             username: appUser._doc.username,
//             email: appUser._doc.email,
//             userType: appUser._doc.userType,
//             status: appUser._doc.status,
//             roleId: appUser._doc.role,
//           },
//           process.env.JWT_SECRET,
//           { expiresIn: "365d" }
//         );

//         await AuthTokenSchema.create({
//           userId: savedUser?._id,
//           token: token,
//         });

//         return sendResponse(
//           res,
//           StatusCode.OK,
//           messages.signIn.success,
//           {
//             ...appUser._doc,
//           },
//           token
//         );
//       }
//     }
//   } catch (error) {
//     console.error("Error during Google Authentication:", error);
//     return sendResponse(res, StatusCode.DELETED, messages.internalServerError);
//   }
// };
