import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Faqs } from "../entities/faqs.entity.js";
import { validateFaqs } from "../validator/validateFaqs.js";

// Add a new Faqs
export const addFaqs = async (req, res) => {
  const { title, subTitle, description, status } = req.body;

  if (!title) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingFaqs = await Faqs.findOne({
      $or: [{ title }],
    });
    if (existingFaqs) {
      return sendResponse(res, StatusCode.BADREQUEST, messages.faqs.faqExits);
    } else {
      const { error } = validateFaqs(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newFaqsData = new Faqs({
          title,
          subTitle,
          description,
          status,
        });
        const savedFaqsData = await newFaqsData.save();

        return sendResponse(res, StatusCode.CREATE, messages.faqs.success, {
          data: savedFaqsData,
        });
      }
    }
  } catch (error) {
    logger.error(`Getting error while add faqs : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Faqs
export const updateFaqs = async (req, res) => {
  const { title, subTitle, description, status, _id } = req.body;
  const updatedFaqsData = { title, subTitle, description, status };
  try {
    const { error } = validateFaqs(updatedFaqsData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedFaqs = await Faqs.findByIdAndUpdate(
        { _id: _id },
        { title, subTitle, description, status },
        { new: true }
      );
      if (!updatedFaqs) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.faqs.faqNotFound
        );
      }

      return sendResponse(res, StatusCode.OK, messages.faqs.faqUpdated, {
        data: updatedFaqs,
      });
    }
  } catch (error) {
    logger.error(`Getting error while  update faqs : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all Faqs
export const getFaqs = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ title: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Faqs, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get faqs : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Faqs
export const deleteFaqs = async (req, res) => {
  const { id } = req.params;
  try {
    await Faqs.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.faqs.faqDeleted);
  } catch (error) {
    logger.error(`Getting error while  delete faqs : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Faqs By Id
export const getFaqsById = async (req, res) => {
  try {
    const faq = await Faqs.findOne({ _id: req.params.id });
    if (!faq) {
      return sendResponse(res, StatusCode.NOT_FOUND, messages.faqs.faqNotFound);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: faq,
    });
  } catch (error) {
    logger.error(`Getting error while  get faq by id : ${error}`);

    console.error("Error fetching faq:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
