import {
  getTokenUserInfo,
  sendResponse,
} from "../../../service/common.service.js";
import { sendEmail } from "../../../service/email.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { FeedbackSchema } from "../entities/feedback-entities.js";
import { validateFeedback } from "../validator/validateFeedback.js";

export const sendFeedback = async (req, res) => {
  const { feedback, rating } = req.body;
  const userInfo = getTokenUserInfo(req);
  // Check if feedback and rating are provided
  if (!feedback || !rating) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    // Validate feedback input
    const { error } = validateFeedback(req.body);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    }

    // Check if feedback from this user already exists
    const existingFeedback = await FeedbackSchema.findOne({
      user: userInfo._id,
    });

    if (existingFeedback) {
      // Update existing feedback
      existingFeedback.feedback = feedback;
      existingFeedback.rating = rating;
      const updatedFeedback = await existingFeedback.save();

      return sendResponse(
        res,
        StatusCode.OK,
        messages.feedback.updated,
        updatedFeedback
      );
    } else {
      // Create a new feedback entry
      const newFeedbackData = new FeedbackSchema({
        feedback,
        rating,
        user: userInfo._id,
      });

      // Save the feedback to the database
      const savedFeedback = await newFeedbackData.save();

      // Optionally, send an email notification to the user or admin
      const emailSent = await sendEmail({
        from: process.env.SENDER_EMAIL,
        to: userInfo.email,
        subject: "Thank you for your feedback",
        text: `We have received your feedback: ${savedFeedback.feedback}`,
        html: `<p>We have received your feedback: ${savedFeedback.feedback}</p><p>Rating: ${savedFeedback.rating}</p>`,
      });

      if (!emailSent) {
        return sendResponse(
          res,
          StatusCode.INTERNAL_SERVER_ERROR,
          "Error sending feedback confirmation email"
        );
      }

      // Send a success response
      return sendResponse(
        res,
        StatusCode.CREATE,
        messages.feedback.success,
        savedFeedback
      );
    }
  } catch (error) {
    logger.error(`Getting error while creating or updating feedback: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
