import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Group } from "../entities/group.entities.js";
import { validateGroup } from "../validator/validateGroup.js";

export const createGroup = async (req, res) => {
  const {
    groupName,
    ageGroup,
    subscriptionPlan,
    transcribeUsageRange,
    users,
    status,
  } = req.body;

  if (
    !groupName ||
    !ageGroup ||
    !subscriptionPlan ||
    !transcribeUsageRange ||
    !users
  ) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingGroup = await Group.findOne({
      $or: [{ groupName }],
    });
    if (existingGroup) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.group.groupExits
      );
    } else {
      const { error } = validateGroup(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newGroupData = new Group({
          groupName,
          ageGroup,
          subscriptionPlan,
          transcribeUsageRange,
          users,
          status,
        });
        const savedGroup = await newGroupData.save();
        await savedGroup.populate("users", "username");
        return sendResponse(res, StatusCode.CREATE, messages.group.success, {
          data: savedGroup,
        });
      }
    }
  } catch (error) {
    console.log("Getting error while create group", error);
    logger.error(`Getting error while create group : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Group
export const updateGroup = async (req, res) => {
  const {
    groupName,
    ageGroup,
    subscriptionPlan,
    transcribeUsageRange,
    users,
    status,
    _id,
  } = req.body;
  const updatedGroupData = {
    groupName,
    ageGroup,
    subscriptionPlan,
    transcribeUsageRange,
    users,
    status,
  };
  try {
    const { error } = validateGroup(updatedGroupData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedGroup = await Group.findByIdAndUpdate(
        { _id: _id },
        {
          groupName,
          ageGroup,
          subscriptionPlan,
          transcribeUsageRange,
          users,
          status,
        },
        { new: true }
      );
      if (!updatedGroup) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.group.groupNotFound
        );
      }

      return sendResponse(res, StatusCode.OK, messages.group.groupUpdated, {
        data: updatedGroup,
      });
    }
  } catch (error) {
    logger.error(`Getting error while update group : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getGroups = async (req, res) => {
  try {
    const { search, page, limit, role } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ groupName: new RegExp(search, "i") }],
      };
    }

    let filters = {};
    if (role) filters.role = role;

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      limit: parseInt(limit) || 10,
    };
    const {
      data: groups,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Group, options);

    const populatedGroup = await Group.populate(groups, {
      path: "users",
      select: "username",
    });

    if (!groups.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: populatedGroup,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while get groups : ${error}`);

    console.error("Error fetching groups:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getGroupsWithOutPagination = async (req, res) => {
  try {
    const data = await Group.find();
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data,
    });
  } catch (error) {
    logger.error(
      `Getting error while get groups with out pagination : ${error}`
    );

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Group
export const deleteGroup = async (req, res) => {
  const { id } = req.params;
  try {
    await Group.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.group.groupDeleted);
  } catch (error) {
    logger.error(`Getting error while delete group : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Group By Id
export const getGroupById = async (req, res) => {
  try {
    const group = await Group.findOne({
      _id: req.params.id,
    });
    if (!group) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.group.groupNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: group,
    });
  } catch (error) {
    logger.error(`Getting error while get group by id : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
