import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { inputFile } from "../entities/inputFileSchema.js";
import { outPutFile } from "../entities/outPutFileSchema.js";
import { getVideoDurationInSeconds } from "get-video-duration";
import { createClient } from "@deepgram/sdk";
import * as fs from "fs";
import mongoose from "mongoose";
import youtubedl from "youtube-dl-exec";
import { fileURLToPath } from "url";
import { dirname } from "path";
import path from "path";
import OpenAI from "openai";
import { thematicAnalysis } from "../entities/thematicAnalysisSchema.js";
import { io } from "../../../index.js";
import { UserSocketMap } from "../../../service/socket.service.js";

// =========================== ADD INPUT FILE =========================== /

const handleSocketMessage = (userId, eventName, data) => {
  if (UserSocketMap[userId]) {
    io.to(UserSocketMap[userId]).emit(eventName, data);
  }
};

export const add = async (req, res) => {
  let userId = req?.user?._id;
  let createInputFile = null;
  let existingEntry = null;
  let initialFileData = null;
  try {
    const { model, transcriptionId, fileUniqueId, fileName, duration } =
      req.body;

    if (!req.files) {
      handleSocketMessage(userId, "upload-error", {
        statusCode: StatusCode.BADREQUEST,
        message: messages.allFieldsRequired,
      });
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.allFieldsRequired
      );
    }

    const files = Array.isArray(req.files) ? req.files : [req.files];

    // Validate file sizes early
    if (files.some((file) => file.size > 209715200)) {
      handleSocketMessage(userId, "upload-error", {
        statusCode: StatusCode.BADREQUEST,
        message:
          "Failed to upload file, Your file is larger than the 200MB size limit.",
      });
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        "Failed to upload file, Your file is larger than the 200MB size limit."
      );
    }

    // Create initial document data
    initialFileData = {
      _id: new mongoose.Types.ObjectId(),
      status: "Processing",
      fileName,
      duration: Math.ceil(duration),
      model: [],
      totalAmount: 0,
    };

    // First check if document exists
    existingEntry = await inputFile.findOne({ transcriptionId });

    // Handle document creation/update based on existence
    if (existingEntry) {
      createInputFile = await inputFile.findOneAndUpdate(
        { transcriptionId },
        {
          $push: { file: initialFileData },
          amount: 0,
        },
        { new: true }
      );
    } else {
      createInputFile = await inputFile.create({
        file: [initialFileData],
        userId: new mongoose.Types.ObjectId(userId),
        amount: 0,
        transcriptionId,
      });
    }

    // Emit socket event if socket exists
    handleSocketMessage(userId, "initial-data-loaded");
    let totalAmount = 0;

    // Process files with better error handling
    const processFile = async (file) => {
      try {
        const filePath = file.path.replace(/\\/g, "/");
        const fileURL = filePath.replace("public/", process.env.FILE_URL);

        // Get duration and process through Deepgram concurrently
        const [deepgramResponse, durationInSeconds] = await Promise.all([
          postDataToDeepgram(filePath, model),
          getVideoDurationInSeconds(filePath),
        ]).catch((err) => console.log(err));
        if (
          !deepgramResponse ||
          deepgramResponse.StatusCode !== StatusCode.OK
        ) {
          // handleSocketMessage(userId, "upload-error", {
          //   statusCode: StatusCode.BADREQUEST,
          //   message: "Transcribing Failed!",
          //   data: {
          //     fileId: createInputFile?._id,
          //     _id: initialFileData?._id,
          //   },
          // });
          throw new Error("Deepgram processing failed");
        }

        // Get Deepgram usage
        const deepgramUsages = { data: 0 };
        // const deepgramUsages = await checkDeepgramUsages(
        //   deepgramResponse?.data?.metadata?.request_id
        // );

        // if (!deepgramUsages || !deepgramUsages.data) {
        //   handleSocketMessage(userId, "upload-error", {
        //     statusCode: StatusCode.BADREQUEST,
        //     message: "Transcribing Failed!",
        //     data: {
        //       fileId: createInputFile?._id,
        //       _id: initialFileData?._id,
        //     },
        //   });
        //   throw new Error("Failed to get Deepgram usage data");
        // }

        const deepgramTranscript =
          deepgramResponse?.data?.results?.channels[0]?.alternatives[0]
            ?.transcript;

        // Run both openAIResponse calls concurrently
        const [summaryResponse, keyThemesResponse] = await Promise.all([
          openAIResponse(deepgramTranscript, "summary"),
          openAIResponse(deepgramTranscript, "keyThemes"),
        ]).catch((error) => {
          handleSocketMessage(userId, "upload-error", {
            statusCode: StatusCode.BADREQUEST,
            message: "Failed to get Summary or Key Themes",
          });
        });

        totalAmount += deepgramUsages.data;

        return {
          _id: initialFileData?._id,
          fileName: file.originalname,
          duration: Math.ceil(durationInSeconds),
          model,
          status: "Completed",
          fileURL,
          deepgram: deepgramResponse.data,
          summary: summaryResponse.data.choices[0].message.content.trim(),
          keyThemes: keyThemesResponse.data.choices[0].message.content.trim(),
          fileId: createInputFile._id,
          transcriptionId,
          fileUniqueId,
        };
      } catch (error) {
        // handleSocketMessage(userId, "upload-error", {
        //   statusCode: StatusCode.BADREQUEST,
        //   message: `Error processing file ${file.originalname}:`,
        //   data: {
        //     fileId: createInputFile?._id,
        //     _id: initialFileData?._id,
        // fileUniqueId
        //   },
        // });
        console.error(`Error processing file ${file.originalname}:`, error);
        throw error;
      }
    };

    // Process files and handle errors
    const [processedFile] = await Promise.all(files.map(processFile));

    if (!processedFile) {
      handleSocketMessage(userId, "upload-error", {
        statusCode: StatusCode.BADREQUEST,
        message: "File processing failed",
        data: {
          fileId: createInputFile?._id,
          _id: initialFileData?._id,
          fileUniqueId,
        },
      });
      throw new Error("File processing failed");
    }

    // Update document with processed data
    const updatedDoc = await inputFile.findOneAndUpdate(
      {
        transcriptionId,
        "file._id": initialFileData?._id,
      },
      {
        $set: {
          "file.$": processedFile,
          amount: Number(totalAmount.toFixed(3)),
        },
      },
      { new: true }
    );

    if (!updatedDoc) {
      handleSocketMessage(userId, "upload-error", {
        statusCode: StatusCode.BADREQUEST,
        message: "Failed to process file",
        data: {
          fileId: createInputFile?._id,
          _id: initialFileData?._id,
          fileUniqueId,
        },
      });
      throw new Error("Failed to update document with processed data");
    }

    // Emit socket event if socket exists
    handleSocketMessage(userId, "upload-success", {
      message: "File uploaded successfully",
      data: processedFile,
    });

    return sendResponse(
      res,
      StatusCode.CREATE,
      "File uploaded successfully",
      processedFile
    );
  } catch (error) {
    // If there was an error and we created a document, clean it up
    if (createInputFile) {
      try {
        await inputFile.deleteOne({ _id: createInputFile._id });
      } catch (cleanupError) {
        console.error("Error cleaning up after failure:", cleanupError);
      }
    }

    console.error("Detailed error:", error);
    logger.error(
      `Error while uploading and transcribing file: ${error.message}\nStack: ${error.stack}`
    );

    handleSocketMessage(userId, "upload-error", {
      statusCode: StatusCode.INTERNAL_SERVER_ERROR,
      message: "Failed to process file",
      data: {
        fileId: createInputFile?._id,
        _id: initialFileData?._id,
        fileUniqueId,
      },
    });
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      error.message || messages.internalServerError
    );
  }
};

//============================ UPDATE INPUT FILES========================//
export const editInputFile = async (req, res) => {
  try {
    const { batchName, _id } = req.body;
    let isInputFile = await inputFile.findById(_id);
    if (!isInputFile) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.inputFile.invalidInputFile
      );
    }
    let updatedInputFile = await inputFile.findByIdAndUpdate(
      _id,
      { batchName: batchName },
      { new: true }
    );
    return sendResponse(
      res,
      StatusCode.OK,
      messages.inputFile.updateSuccess,
      updatedInputFile
    );
  } catch (error) {
    console.log("Getting error while edit input file ::", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== GET INPUT FILE =========================== /
export const get = async (req, res) => {
  try {
    const { take, page, search, isBatch } = req.query;

    delete req.query.take;
    delete req.query.page;
    delete req.query.search;
    delete req.query.isBatch;

    if (req.user.userType != "user") {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.insufficient_Permissions
      );
    }

    // PAGINATION
    const takePerPage = Number(take) || 10;
    let pageCount = Number(page) || 1;
    if (pageCount == 0) {
      pageCount = 1;
    }
    const skip = (pageCount - 1) * takePerPage;

    // CREATE QUERY
    let query = {};

    // GLOBAL FILTER
    if (search) {
      query = {
        ...query,
        $and: [
          {
            $or: [
              { "file.fileName": { $regex: search, $options: "i" } },
              { batchName: { $regex: search, $options: "i" } },
            ],
          },
        ],
      };
    }

    if (isBatch === "true") {
      query.batchName = { $ne: null };
    } else if (isBatch === "false") {
      query.batchName = { $eq: null };
    }

    const fileInputResponse = await inputFile.aggregate([
      {
        $match: query,
      },
      {
        $sort: {
          createdAt: -1,
        },
      },
      {
        $match: {
          userId: new mongoose.Types.ObjectId(req?.user?._id),
        },
      },
      {
        $facet: {
          metadata: [{ $count: "totalCount" }],
          data: [{ $skip: skip }, { $limit: takePerPage }],
        },
      },
      // Project the fields you need from the `data` array, removing `deepgram` from each file object
      {
        $project: {
          metadata: 1,
          data: {
            $map: {
              input: "$data",
              as: "doc",
              in: {
                _id: "$$doc._id",
                batchName: "$$doc.batchName",
                amount: "$$doc.amount",
                createdAt: "$$doc.createdAt",
                updatedAt: "$$doc.updatedAt",
                userId: "$$doc.userId",
                status: "$$doc.status",
                summary: "$$doc.summary",
                transcriptionId: "$$doc.transcriptionId",
                file: {
                  $map: {
                    input: "$$doc.file",
                    as: "fileDoc",
                    in: {
                      _id: "$$fileDoc._id",
                      fileId: "$$fileDoc.fileId",
                      fileName: "$$fileDoc.fileName",
                      fileURL: "$$fileDoc.fileURL",
                      duration: "$$fileDoc.duration",
                      status: "$$fileDoc.status",
                      // Exclude deepgram field here by simply not listing it
                    },
                  },
                },
              },
            },
          },
        },
      },
    ]);

    const metadata = fileInputResponse[0].metadata[0];
    const totalCount = metadata ? metadata.totalCount : 0;

    // Pagination
    const totalPages = Math.ceil(totalCount / takePerPage);
    const nextPage = pageCount < totalPages ? pageCount + 1 : null;
    const prevPage = pageCount > 1 ? pageCount - 1 : null;

    const responseData = {
      docs: fileInputResponse[0]?.data || [],
      count: totalCount,
      pages: totalPages,
      currentPage: Number(page),
      nextPage,
      prevPage,
    };

    return sendResponse(
      res,
      StatusCode.OK,
      "Files data fetch successfully.",
      responseData
    );
  } catch (error) {
    console.log("Getting error while get the input file", error);
    logger.error(`Getting error while get the input file : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== GENERATE BATCH =========================== /
async function generateBatch(userId) {
  try {
    const count = await inputFile.countDocuments({
      userId: userId,
      batchName: { $ne: null },
    });

    return `Batch-${count}`;
  } catch (error) {
    console.log("Getting error while generate batch", error);
    logger.error(`Getting error while add-user : ${error}`);

    throw new Error("Getting error while generate batch");
  }
}

// =========================== POST DATA TO DEEPGRAM TRANSCRIBE =========================== /
async function postDataToDeepgram(url, model) {
  try {
    const deepgram = createClient(process.env.DEEPGRAM_KEY);

    const transcribeOptions = {
      model: "nova-2",
      smart_format: true,
      summarize: "v2",
      topics: true, //model.includes("TopicDetection") ? true : false,
      // diarize: model.includes("Diarization") ? true : false,
      // paragraphs: model.includes("Paragraphs") ? true : false,
    };

    const { result, error } = await deepgram.listen.prerecorded.transcribeFile(
      fs.createReadStream(url),
      transcribeOptions
    );

    if (error) {
      console.log("error in post data to deepgram", error);
      return { StatusCode: StatusCode.BADREQUEST };
    }
    return { StatusCode: StatusCode.OK, data: result };
  } catch (error) {
    console.log("error in post data to deepgram", error);
    return { StatusCode: StatusCode.BADREQUEST };
  }
}

// =========================== DELETE INPUT FILE & BATCH =========================== /
export const deleteFiles = async (req, res) => {
  try {
    if (
      !Array.isArray(req.body.ids) ||
      req.body.ids.length === 0 ||
      req.body.ids.every((id) => id === null)
    ) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        "Please select files to delete."
      );
    }

    const _id = req.body.ids.map((id) => new mongoose.Types.ObjectId(id));

    const filesToDelete = await inputFile.find({ _id: { $in: _id } });

    if (!filesToDelete || filesToDelete.length === 0) {
      return sendResponse(res, StatusCode.BADREQUEST, "Invalid Id");
    }

    filesToDelete.forEach((doc) => {
      if (Array.isArray(doc.file)) {
        doc.file.forEach((fileObj) => {
          if (fileObj.fileName) {
            const url = fileObj.fileURL;

            // Use regex to split by either "/" or "\" around "input-file"
            const parts = url?.split(/[/\\]input-file[/\\]/) ?? [];
            const fileName = parts[1] ?? fileObj.fileName;

            const VIDEO_PATH = "public/input-file";
            const filePath = `${VIDEO_PATH}/${fileName}`;

            // Check if the file exists before attempting to delete it
            if (fs.existsSync(filePath)) {
              fs.unlinkSync(filePath);
            } else {
              console.warn(`File not found, skipping deletion: ${filePath}`);
            }
          } else {
            console.error("Filename is undefined in the file object:", fileObj);
          }
        });
      } else {
        console.error("No files associated with this document:", doc);
      }
    });

    // Delete the document entries in the database after processing files
    await inputFile.deleteMany({ _id: { $in: _id } });

    return sendResponse(res, StatusCode.OK, "Files deleted successfully.");
  } catch (error) {
    console.log("Error while deleting the input files", error);
    logger.error(`Error while deleting the input files: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== INPUT FILE GET BY ID =========================== /
export const getById = async (req, res) => {
  try {
    const _id = req.params.id;

    const findInputFile = await inputFile.aggregate([
      {
        $match: {
          _id: new mongoose.Types.ObjectId(_id),
        },
      },
      {
        $lookup: {
          from: "thematic-analyses",
          localField: "_id",
          foreignField: "mediaId",
          as: "thematicAnalyses",
        },
      },
      {
        $project: {
          _id: 1,
          file: 1,
          batchName: 1,
          createdAt: 1,
          userId: 1,
          // thematicAnalyses: "$thematicAnalyses",
          thematicAnalyses: {
            $slice: ["$thematicAnalyses", -1, 1],
          },
        },
      },
    ]);

    if (!findInputFile) {
      return sendResponse(res, StatusCode.NOT_FOUND, "Record not found.");
    }

    return sendResponse(
      res,
      StatusCode.OK,
      "Files data fetch successfully.",
      findInputFile[0]
    );
  } catch (error) {
    console.log("Getting error while fine result by id the input file", error);
    logger.error(
      `Getting error while fine result by id the input file : ${error}`
    );

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== YOUTUBE URL TO TRANSCRIBE =========================== /
const processYouTubeURL = async (ytURL, userId, data) => {
  let totalAmount = 0;
  let model = [];
  const title = await youtubedl(ytURL, {
    getTitle: true,
    noWarnings: true,
    noCheckCertificate: true,
    preferFreeFormats: true,
    youtubeSkipDashManifest: true,
  });
  const fileName = `${title
    .replace(/[^a-z0-9]/gi, "_")
    .toLowerCase()}_${Date.now()}`;
  const __filename = fileURLToPath(import.meta.url);
  const __dirname = dirname(__filename);
  const outputPath = path.join(
    __dirname,
    "../../../../public/input-file",
    `${fileName}.mp3`
  );

  await youtubedl(ytURL, {
    output: outputPath,
    extractAudio: true,
    audioFormat: "mp3",
  }).catch((err) => {
    handleSocketMessage(userId, "upload-error", {
      statusCode: StatusCode.BADREQUEST,
      message: "Failed to upload the file",
      data,
    });
  });

  const duration = await getVideoDurationInSeconds(
    outputPath.replace(/\\/g, "/")
  ).catch((err) => {
    handleSocketMessage(userId, "upload-error", {
      statusCode: StatusCode.BADREQUEST,
      message: "Failed to process the file",
      data,
    });
    throw err;
  });

  const deepgramResponse = await postDataToDeepgram(
    outputPath.replace(/\\/g, "/"),
    model
  ).catch((err) => {
    handleSocketMessage(userId, "upload-error", {
      statusCode: StatusCode.BADREQUEST,
      message: "Failed to process the file",
      data,
    });
    throw err;
  });
  const fileURL = outputPath.replace(
    "/var/www/html/summary-ai-backend/public/",
    process.env.FILE_URL
  );

  if (deepgramResponse.StatusCode === StatusCode.OK) {
    const deepgramUsages = { data: 0 };
    // const deepgramUsages = await checkDeepgramUsages(
    //   deepgramResponse?.data?.metadata?.request_id
    // );

    const deepgramTranscript =
      deepgramResponse?.data?.results?.channels[0]?.alternatives[0]?.transcript;

    // Run both openAIResponse calls concurrently
    const [summaryResponse, keyThemesResponse] = await Promise.all([
      openAIResponse(deepgramTranscript, "summary"),
      openAIResponse(deepgramTranscript, "keyThemes"),
    ]).catch((err) => {
      handleSocketMessage(userId, "upload-error", {
        statusCode: StatusCode.BADREQUEST,
        message: "Failed to get Summary or Key Themes",
      });
      throw err;
    });

    totalAmount += deepgramUsages.data;
    return {
      // _id: new mongoose.Types.ObjectId(),
      fileName: title,
      duration: Math.ceil(duration),
      model,
      status: "Completed",
      fileURL,
      deepgram: deepgramResponse.data,
      summary: summaryResponse.data.choices[0].message.content.trim(),
      keyThemes: keyThemesResponse.data.choices[0].message.content.trim(),
      totalAmount,
    };
  } else {
    console.log("Error in deepgramResponse", deepgramResponse);
    handleSocketMessage(userId, "upload-error", {
      statusCode: StatusCode.BADREQUEST,
      message: "Failed to Transcribe the file",
      data,
    });
    throw new Error("Failed to Transcribe the file");
  }
};

export const youTubeUpload = async (req, res) => {
  const {
    youTubeURL,
    transcriptionId,
    fileUniqueId,
    isFolder,
    fileName,
    duration,
  } = req.body;
  let initialFileData = null;
  let existingEntry = null;
  let createInputFile = null;
  let userId = req?.user?._id;
  try {
    // Create initial document with "Processing" status and frontend data
    initialFileData = {
      _id: new mongoose.Types.ObjectId(),
      status: "Processing",
      fileName,
      duration: Math.ceil(duration),
      model: [],
      totalAmount: 0,
    };

    // Determine batchName based on the `isFolder` property
    const batchName = isFolder ? `yt-${await generateBatch(userId)}` : null;

    // Find existing entry or create new one with initial processing status
    existingEntry = await inputFile.findOne({ transcriptionId });
    createInputFile = existingEntry
      ? await inputFile.findOneAndUpdate(
          { transcriptionId },
          {
            $push: { file: initialFileData },
            amount: 0,
          },
          { new: true }
        )
      : await inputFile.create({
          file: [initialFileData],
          batchName,
          userId: new mongoose.Types.ObjectId(userId),
          amount: 0,
          transcriptionId,
        });

    // Emit socket event if socket exists
    handleSocketMessage(userId, "initial-data-loaded");

    // Send immediate response with processing status
    const initialResponse = createInputFile.toObject();
    initialResponse.transcriptionId = transcriptionId;
    initialResponse.fileUniqueId = fileUniqueId;

    // Send initial response
    // sendResponse(
    //   res,
    //   StatusCode.CREATE,
    //   "File upload started",
    //   initialResponse
    // );

    // Process the YouTube URL in the background
    const processedFileData = await processYouTubeURL(youTubeURL, userId, {
      fileId: createInputFile._id,
      _id: initialFileData._id,
      fileUniqueId,
    });
    const wholeObj = {
      ...processedFileData,
      _id: initialFileData?._id,
      fileId: createInputFile._id,
    };

    wholeObj.transcriptionId = transcriptionId;
    wholeObj.fileUniqueId = fileUniqueId;

    // Update the document with processed data
    await inputFile.findOneAndUpdate(
      {
        transcriptionId,
        "file._id": initialFileData?._id,
      },
      {
        $set: {
          "file.$": wholeObj,
          amount: Number(processedFileData.totalAmount.toFixed(3)),
        },
      }
    );

    const createInputFileResponse = createInputFile.toObject();
    createInputFileResponse.transcriptionId = transcriptionId;
    createInputFileResponse.fileUniqueId = fileUniqueId;

    handleSocketMessage(userId, "upload-success", {
      message: "File uploaded successfully",
      data: wholeObj,
    });

    return sendResponse(
      res,
      StatusCode.CREATE,
      "File uploaded successfully",
      wholeObj
    );
  } catch (error) {
    console.log("Error while uploading YouTube link:", error);
    logger.error(`Error while uploading YouTube link: ${error}`);

    // handleSocketMessage(userId, "upload-error", {
    //   statusCode: StatusCode.BADREQUEST,
    //   message: "Failed to process file",
    //   data: {
    //     fileId: createInputFile?._id,
    //     _id: initialFileData?._id,
    //   },
    // });

    // Update status to Failed if there's an error
    if (initialFileData?._id) {
      await inputFile.findOneAndUpdate(
        {
          transcriptionId,
          "file._id": initialFileData?._id,
        },
        {
          $set: {
            "file.$.status": "Failed",
            "file.$.error": error.message,
          },
        }
      );
    }

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== CREATE BATCH WHEN MULTIPLE FILES =========================
export const createBatch = async (req, res) => {
  const { transcriptionId, isFile, file } = req.body;
  const batchName = isFile
    ? await generateBatch(req?.user?._id)
    : `yt-${await generateBatch(req?.user?._id)}`;

  try {
    let createdData = await inputFile.create({
      file,
      batchName,
      userId: new mongoose.Types.ObjectId(req?.user?._id),
      amount: 0,
      transcriptionId,
    });

    sendResponse(
      res,
      StatusCode.CREATE,
      "Batch created successfully",
      createdData
    );
  } catch (error) {
    console.log("Error while creating batch:", error);
    logger.error(`Error while creating batch: ${error}`);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== OUT PUT FILE =========================== /
export const exportOutPutFile = async (req, res) => {
  try {
    const { _id, exportType } = req.body;

    if (!_id || !exportType) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        "Id & export type is required"
      );
    }

    const findInputFile = await inputFile.findById({
      _id,
    });

    if (!findInputFile) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        "Please check your selected Id"
      );
    }

    const saveOutPutFile = await outPutFile.create({
      file: findInputFile.file,
      batchName: findInputFile.batchName,
      userId: new mongoose.Types.ObjectId(req?.user?._id),
      exportType: exportType,
      inputFileId: new mongoose.Types.ObjectId(findInputFile?._id),
    });

    return sendResponse(
      res,
      StatusCode.CREATE,
      "Out-Put file store successfully",
      saveOutPutFile
    );
  } catch (error) {
    console.log("Getting Error while store the out put file:", error);
    logger.error(`Getting Error while store the out put file: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getAllOutPutFile = async (req, res) => {
  try {
    const { take, page, search, isBatch } = req.query;

    delete req.query.take;
    delete req.query.page;
    delete req.query.search;
    delete req.query.isBatch;

    if (req.user.userType != "user") {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.insufficient_Permissions
      );
    }

    // PAGINATION
    const takePerPage = Number(take) || 10;
    let pageCount = Number(page) || 1;
    if (pageCount == 0) {
      pageCount = 1;
    }
    const skip = (pageCount - 1) * takePerPage;

    // CREATE QUERY
    let query = {};

    // GLOBAL FILTER
    if (search) {
      query = {
        ...query,
        $and: [
          {
            $or: [
              { "file.fileName": { $regex: search, $options: "i" } },
              { batchName: { $regex: search, $options: "i" } },
            ],
          },
        ],
        userId: new mongoose.Types.ObjectId(req?.user?._id),
      };
    }

    if (isBatch === "true") {
      query.batchName = { $ne: null };
    } else if (isBatch === "false") {
      query.batchName = { $eq: null };
    }

    const fileInputResponse = await outPutFile.aggregate([
      {
        $match: query,
      },
      {
        $sort: {
          createdAt: -1,
        },
      },
      {
        $match: {
          userId: new mongoose.Types.ObjectId(req?.user?._id),
        },
      },
      {
        $facet: {
          metadata: [{ $count: "totalCount" }],
          data: [{ $skip: skip }, { $limit: takePerPage }],
        },
      },
      // Remove `deepgram` field here using $project
      {
        $project: {
          metadata: 1,
          data: {
            $map: {
              input: "$data",
              as: "doc",
              in: {
                _id: "$$doc._id",
                batchName: "$$doc.batchName",
                userId: "$$doc.userId",
                exportType: "$$doc.exportType",
                inputFileId: "$$doc.inputFileId",
                createdAt: "$$doc.createdAt",
                file: {
                  $map: {
                    input: "$$doc.file",
                    as: "fileDoc",
                    in: {
                      _id: "$$fileDoc._id",
                      fileName: "$$fileDoc.fileName",
                      duration: "$$fileDoc.duration",
                      model: "$$fileDoc.model",
                      status: "$$fileDoc.status",
                      fileURL: "$$fileDoc.fileURL",
                      // Do not include `deepgram` field
                    },
                  },
                },
              },
            },
          },
        },
      },
    ]);

    const metadata = fileInputResponse[0].metadata[0];
    const totalCount = metadata ? metadata.totalCount : 0;

    // PAGINATION
    const totalPages = Math.ceil(totalCount / takePerPage);
    const nextPage = pageCount < totalPages ? pageCount + 1 : null;
    const prevPage = pageCount > 1 ? pageCount - 1 : null;

    const responseData = {
      docs: fileInputResponse[0]?.data || [],
      count: totalCount,
      pages: totalPages,
      currentPage: Number(page),
      nextPage,
      prevPage,
    };

    return sendResponse(
      res,
      StatusCode.OK,
      "Out-Put Files data fetch successfully.",
      responseData
    );
  } catch (error) {
    console.log("Getting Error while get the all out put file:", error);
    logger.error(`Getting Error while get the all out put file: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== DELETE OUTPUT FILE & BATCH =========================== /
export const deleteOutPutFiles = async (req, res) => {
  try {
    if (
      !Array.isArray(req.body.ids) ||
      req.body.ids.length === 0 ||
      req.body.ids.every((id) => id === null)
    ) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        "Please select to delete input files."
      );
    }
    const _id = req.body.ids.map((id) => new mongoose.Types.ObjectId(id));

    await outPutFile.deleteMany({ _id: { $in: _id } });

    return sendResponse(
      res,
      StatusCode.OK,
      "Output files deleted successfully."
    );
  } catch (error) {
    console.log("Getting error while delete the output file", error);
    logger.error(`Getting error while delete the output file : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// =========================== THEMATIC ANALYTIC =========================== /
export const thematicAnalytic = async (req, res) => {
  try {
    const openai = new OpenAI({
      apiKey: process.env.THEMATIC_ANALYTIC_KEY,
    });
    const { _id, content } = req.body;

    if (!_id) {
      return sendResponse(res, StatusCode.BADREQUEST, "Id is required.");
    }

    for (let index = 0; index < content.length; index++) {
      const element = content[index];
      let len = element.length;

      if (len > 128000) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          "Your request entity too large."
        );
      }
    }

    if (!content) {
      return sendResponse(res, StatusCode.BADREQUEST, "Content is required.");
    }

    const response = await openai.chat.completions.create({
      model: "chatgpt-4o-latest",
      messages: [...content],
      max_tokens: 16383,
      temperature: 0.3,
      top_p: 1.0,
      frequency_penalty: 0.0,
      presence_penalty: 0.0,
    });

    if (response.choices[0].message.content.trim()) {
      await thematicAnalysis.create({
        mediaId: new mongoose.Types.ObjectId(_id),
        userId: new mongoose.Types.ObjectId(req?.user?._id),
        response: response.choices[0].message.content.trim(),
      });
    }

    return sendResponse(
      res,
      StatusCode.OK,
      messages.success,
      response.choices[0].message.content.trim()
    );
  } catch (error) {
    console.log("Getting error while thematic analytic processing", error);
    logger.error(`Getting error while thematic analytic processing: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

async function checkDeepgramUsages(requestId) {
  try {
    const url = `https://api.deepgram.com/v1/projects/${process.env.DEEPGRAM_PROJECT_ID}/requests/${requestId}`;
    const options = {
      method: "GET",
      headers: {
        accept: "application/json",
        Authorization: `Token ${process.env.DEEPGRAM_KEY}`,
      },
    };

    const response = await fetch(url, options).catch((err) => console.log(err));
    const json = await response.json();

    const usd = json.response?.details?.usd ?? 0;

    return { StatusCode: StatusCode.OK, data: usd };
  } catch (error) {
    console.log("Getting error while checking deepgram usage", error);
    logger.error(`Getting error while checking deepgram usage: ${error}`);

    throw new Error("Getting error while checking deepgram usage");
  }
}

async function openAIResponse(content, isFlag) {
  try {
    const openai = new OpenAI({
      apiKey: process.env.THEMATIC_ANALYTIC_KEY,
    });
    let contentMsg;

    if (isFlag === "summary") {
      contentMsg = [
        {
          role: "user",
          content: `${content}\n You are ChatGPT-4o-latest, a large language model trained by OPENAI, based on chatgpt-4o-latest architecture, Please consider ,I have pasted a transcribed notes from 1 message/speech below: Your task, do a detailed review of the notes and Summerise the message / speech calling out key references`,
        },
      ];
    } else if (isFlag === "keyThemes") {
      contentMsg = [
        {
          role: "user",
          content: `${content}\n You are ChatGPT-4o-latest, a large language model trained by OPENAI, based on chatgpt-4o-latest architecture, Please consider ,I have pasted a transcribed notes from 1 message/speech below: Your task, do a detailed review of the notes and give key Themes with references in tabular format`,
        },
      ];
    }

    const response = await openai.chat.completions.create({
      model: "chatgpt-4o-latest",
      messages: contentMsg,
      max_tokens: 16383,
      temperature: 0.3,
      top_p: 1.0,
      frequency_penalty: 0.0,
      presence_penalty: 0.0,
    });

    return { StatusCode: StatusCode.OK, data: response };
  } catch (error) {
    console.log("Getting error while openAI API is processing", error);
    logger.error(`Getting error while openAI API is processing: ${error}`);

    throw new Error("Getting error while openAI API is processing");
  }
}
