import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Language } from "../entities/languageSetup.entity.js";
import { validateLanguage } from "../validator/validateLanguage.js";

// Add a new language
export const addLanguage = async (req, res) => {
  const { newLanguage, status } = req.body;

  if (!newLanguage) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingUser = await Language.findOne({
      $or: [{ newLanguage }],
    });
    if (existingUser) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.language.languageExits
      );
    } else {
      const { error } = validateLanguage(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newLanguageData = new Language({ newLanguage, status });
        const savedLanguageData = await newLanguageData.save();

        return sendResponse(res, StatusCode.CREATE, messages.language.success, {
          data: savedLanguageData,
        });
      }
    }
  } catch (error) {
    logger.error(`Getting error while add language : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing language
export const updateLanguage = async (req, res) => {
  const { newLanguage, status, _id } = req.body;
  const updatedLanguageData = { newLanguage, status };
  try {
    const { error } = validateLanguage(updatedLanguageData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedLanguage = await Language.findByIdAndUpdate(
        { _id: _id },
        { newLanguage, status },
        { new: true }
      );
      if (!updatedLanguage) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.language.languageNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.language.languageUpdated,
        {
          data: updatedLanguage,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while  update language : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all language
export const getLanguages = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ newLanguage: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Language, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get languages : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a language
export const deletelanguage = async (req, res) => {
  const { id } = req.params;
  try {
    await Language.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.language.languageDeleted);
  } catch (error) {
    logger.error(`Getting error while  delete language : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get language By Id
export const getLanguageById = async (req, res) => {
  try {
    const language = await Language.findOne({ _id: req.params.id });
    if (!language) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.language.languageNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: language,
    });
  } catch (error) {
    logger.error(`Getting error while  get language by id : ${error}`);

    console.error("Error fetching user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
