import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { NOTIFICATION, StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Group } from "../../group/entities/group.entities.js";
import { AppUser } from "../../user/entities/userSchema.js";
import {
  AllUsersNotificationSchema,
  Notification,
} from "../entities/notificationSchema.js";
import { validateNotification } from "../validator/validateNotification.js";

// Add a new notification
export const addNotification = async (req, res) => {
  const { title, groupId, userId, message, notificationType } = req.body;

  if (!title || !message || !notificationType) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingNotification = await Notification.findOne({
      $or: [{ title }],
    });
    if (existingNotification) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.notification.notificationExits
      );
    } else {
      const { error } = validateNotification(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newNotificationData = new Notification({
          title,
          userId,
          message,
          notificationType,
          groupId,
        });
        const savedNotification = await newNotificationData.save();
        await savedNotification.populate([
          {
            path: "userId",
            select: "username",
          },

          {
            path: "groupId",
            select: "groupName",
          },
        ]);
        if (savedNotification) {
          if (
            savedNotification.notificationType ==
            NOTIFICATION.INDIVIDUAL_NOTIFICATION
          ) {
            const INDIVIDUAL_NOTIFICATION = new AllUsersNotificationSchema({
              notificationId: savedNotification._id,
              userId: savedNotification.userId,
              isSeen: false,
            });

            await INDIVIDUAL_NOTIFICATION.save();
          } else if (
            savedNotification.notificationType ==
            NOTIFICATION.GROUP_NOTIFICATION
          ) {
            const usersInGroup = await Group.findOne({ _id: groupId });

            const groupNotifications = usersInGroup.users.map((userId) => ({
              notificationId: savedNotification._id,
              userId: userId,
              isSeen: false,
            }));
            await AllUsersNotificationSchema.insertMany(groupNotifications);
          } else {
            const allUsers = await AppUser.find();

            const allUsersNotifications = allUsers.map((item) => ({
              notificationId: savedNotification._id,
              userId: item._id,
              isSeen: false,
            }));
            await AllUsersNotificationSchema.insertMany(allUsersNotifications);
          }
        }

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.notification.success,
          {
            data: savedNotification,
          }
        );
      }
    }
  } catch (error) {
    console.log("Getting error while add notification", error);
    logger.error(`Getting error while add notification : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing notification
export const updateNotification = async (req, res) => {
  const { title, userId, message, notificationType, groupId, _id } = req.body;
  const updatedNotificationData = {
    title,
    userId,
    message,
    notificationType,
    groupId,
  };
  try {
    const { error } = validateNotification(updatedNotificationData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedNotification = await Notification.findByIdAndUpdate(
        { _id: _id },
        {
          title,
          userId,
          message,
          notificationType,
          groupId,
        },
        { new: true }
      );
      if (!updatedNotification) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.notification.notificationNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.notification.notificationUpdated,
        {
          data: updatedNotification,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while  update notification : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all notifications
export const getNotifications = async (req, res) => {
  try {
    const { search, page, take, role } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ title: new RegExp(search, "i") }],
      };
    }

    let filters = {};
    if (role) filters.role = role;

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };
    const {
      data: notifications,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Notification, options);
    const populateNotifications = await Notification.populate(notifications, [
      {
        path: "userId",
        select: "username",
      },

      {
        path: "groupId",
        select: "groupName",
      },
    ]);

    if (!populateNotifications.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: populateNotifications,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get notifications : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a notification
export const deleteNotification = async (req, res) => {
  const { id } = req.params;
  try {
    await Notification.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.notification.notificationDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete notification : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get notification By Id
export const getNotificationById = async (req, res) => {
  try {
    const notification = await Notification.findOne({
      _id: req.params.id,
    });
    if (!notification) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.notification.notificationNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      notification,
    });
  } catch (error) {
    logger.error(`Getting error while notification by id : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
