import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { OutputFormat } from "../entities/outputFormat.entity.js";
import { validateOutputFormat } from "../validator/validateOutputFormat.js";

// Add a new Output Format
export const addOutputFormat = async (req, res) => {
  const { uploadNewFormat, status } = req.body;

  if (!uploadNewFormat) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingOutputFormat = await OutputFormat.findOne({
      $or: [{ uploadNewFormat }],
    });
    if (existingOutputFormat) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.outputFormat.outputFormatExits
      );
    } else {
      const { error } = validateOutputFormat(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newexistingOutputFormatData = new OutputFormat({
          uploadNewFormat,
          status,
        });
        const savedOutputFormatData = await newexistingOutputFormatData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.outputFormat.success,
          {
            data: savedOutputFormatData,
          }
        );
      }
    }
  } catch (error) {
    logger.error(`Getting error while add output format : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Output Format
export const updateOutputFormat = async (req, res) => {
  const { uploadNewFormat, status, _id } = req.body;
  const updatedOutputFormatData = { uploadNewFormat, status };
  try {
    const { error } = validateOutputFormat(updatedOutputFormatData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedOutputFormat = await OutputFormat.findByIdAndUpdate(
        { _id: _id },
        { uploadNewFormat, status },
        { new: true }
      );
      if (!updatedOutputFormat) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.outputFormat.outputFormatNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.outputFormat.outputFormatUpdated,
        {
          data: updatedOutputFormat,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while update output format : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all Output Format
export const getOutputFormats = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ uploadNewFormat: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(OutputFormat, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while get output formats : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Output Format
export const deleteOutputFormat = async (req, res) => {
  const { id } = req.params;
  try {
    await OutputFormat.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.outputFormat.outputFormatDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete output format: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Output Format By Id
export const getOutputFormatById = async (req, res) => {
  try {
    const outputFormat = await OutputFormat.findOne({ _id: req.params.id });
    if (!outputFormat) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.outputFormat.outputFormatNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: outputFormat,
    });
  } catch (error) {
    logger.error(`Getting error while get output format by id: ${error}`);
    console.error("Error fetching user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
