import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Prompts } from "../entities/prompts.entity.js";
import { validatePrompts } from "../validator/validatePrompts.js";

// Add a new Prompt
export const addPrompts = async (req, res) => {
  const { name, prompt, status } = req.body;

  if (!name || !prompt) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingPrompt = await Prompts.findOne({
      $or: [{ name }],
    });
    if (existingPrompt) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.prompts.promptExits
      );
    } else {
      const { error } = validatePrompts(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newPromptData = new Prompts({
          name,
          prompt,
          status,
        });
        const savedPromptData = await newPromptData.save();

        return sendResponse(res, StatusCode.CREATE, messages.prompts.success, {
          data: savedPromptData,
        });
      }
    }
  } catch (error) {
    logger.error(`Getting error while add prompt : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Prompt
export const updatePrompt = async (req, res) => {
  const { name, prompt, status, _id } = req.body;
  const updatedPromptdata = { name, prompt, status };
  try {
    const { error } = validatePrompts(updatedPromptdata);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedPrompt = await Prompts.findByIdAndUpdate(
        { _id: _id },
        { name, prompt, status },
        { new: true }
      );
      if (!updatedPrompt) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.prompts.promptNotFound
        );
      }

      return sendResponse(res, StatusCode.OK, messages.prompts.promptUpdated, {
        data: updatedPrompt,
      });
    }
  } catch (error) {
    logger.error(`Getting error while  update prompt : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all Prompts
export const getPrompts = async (req, res) => {
  try {
    const { search, page, take, isDropdown } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ name: new RegExp(search, "i") }],
      };
    }

    // Check if isDropdown is true
    if (isDropdown === "true") {
      const data = await Prompts.find({ ...searchQuery, status: true }); // Fetch all data where status is true
      if (!data.length) {
        return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
      }
      return sendResponse(res, StatusCode.OK, messages.success, { data });
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Prompts, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get prompts : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Prompt
export const deletePrompt = async (req, res) => {
  const { id } = req.params;
  try {
    await Prompts.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.prompts.promptDeleted);
  } catch (error) {
    logger.error(`Getting error while  delete prompt : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Prompt By Id
export const getPromptById = async (req, res) => {
  try {
    const prompt = await Prompts.findOne({ _id: req.params.id });
    if (!prompt) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.prompts.promptNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: prompt,
    });
  } catch (error) {
    logger.error(`Getting error while  get prompt by id : ${error}`);

    console.error("Error fetching prompt:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
