import {
  getTokenUserInfo,
  sendResponse,
} from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Duration } from "../../delete-file-duration/entities/duration.entitie.js";
import { Role } from "../entities/role.entity.js";
import { validateRole } from "../validator/validateRole.js";

// Add a new role
export const addRole = async (req, res) => {
  const { roleName, permission, status } = req.body;

  if (!roleName || !permission) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    let userInfo = getTokenUserInfo(req);
    const existingUser = await Role.findOne({ $or: [{ roleName }] });
    if (existingUser) {
      return sendResponse(res, StatusCode.BADREQUEST, messages.role.roleExits);
    } else {
      const { error } = validateRole(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newRole = new Role({
          roleName,
          permission,
          status,
          userId: userInfo?._id,
        });
        const savedRole = await newRole.save();

        return sendResponse(res, StatusCode.CREATE, messages.role.success, {
          data: savedRole,
        });
      }
    }
  } catch (error) {
    logger.error(`Getting error while add role : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing role
export const updateRole = async (req, res) => {
  const { roleName, permission, status, _id } = req.body;
  const updatedRoleData = { roleName, permission, status };
  try {
    const { error } = validateRole(updatedRoleData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      let userInfo = getTokenUserInfo(req);

      const updatedRole = await Role.findByIdAndUpdate(
        { _id: _id },
        { roleName, permission, status, userId: userInfo?._id },
        { new: true }
      );
      if (!updatedRole) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.role.roleNotFound
        );
      }

      return sendResponse(res, StatusCode.OK, messages.role.roleUpdated, {
        data: updatedRole,
      });
    }
  } catch (error) {
    logger.error(`Getting error while update role : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all roles
export const getRoles = async (req, res) => {
  try {
    const { search, page, take, role } = req.query;

    let searchQuery = {
      roleName: {
        $nin: ["Super Admin"],
      },
    };
    if (search) {
      searchQuery = {
        $and: [
          { roleName: { $nin: ["Super Admin"] } },
          { roleName: new RegExp(search, "i") },
        ],
      };
    }
    let filters = {};
    if (role) filters.role = role;

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };
    const {
      data: roles,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Role, options);

    if (!roles.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    const rolesWithoutPermission = roles?.map(
      ({ _id, roleName, status, createdAt, updatedAt, __v, ...rest }) => ({
        _id,
        roleName,
        status,
        createdAt,
        updatedAt,
        __v,
      })
    );

    return sendResponse(res, StatusCode.OK, messages.success, {
      // data: {
      data: rolesWithoutPermission,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
        // },
      },
    });
  } catch (error) {
    logger.error(`Getting error while get roles : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getAdminRoles = async (req, res) => {
  try {
    const data = await Role.find({
      roleName: {
        $nin: ["Super Admin"],
      },
    });
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }

    const rolesWithoutPermission = data?.map(
      ({ _id, roleName, status, createdAt, updatedAt, __v, ...rest }) => ({
        _id,
        roleName,
        status,
        createdAt,
        updatedAt,
        __v,
      })
    );
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: rolesWithoutPermission,
    });
  } catch (error) {
    logger.error(`Getting error while get admin roles: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a role
export const deleteRole = async (req, res) => {
  const { id } = req.params;
  try {
    await Role.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.role.roleDeleted);
  } catch (error) {
    logger.error(`Getting error while delete role : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Role By Id
export const getRoleById = async (req, res) => {
  try {
    const role = await Role.findOne({ _id: req.params.id });
    if (!role) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.role.roleNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, { data: role });
  } catch (error) {
    logger.error(`Getting error while get role : ${error}`);

    console.error("Error fetching user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
