import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Subscription } from "../entities/subscriptionSchema.js";
import { validateSubscription } from "../validator/validateSubscription.js";

// Add a new subscription
export const addSubscription = async (req, res) => {
  const {
    subscriptionTitle,
    status,
    planDuration,
    subscriptionAmount,
    startDate,
    endDate,
    subscriptionContent,
  } = req.body;

  if (
    !subscriptionTitle ||
    !planDuration ||
    !subscriptionAmount ||
    !startDate ||
    !endDate ||
    !subscriptionContent
  ) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingSubscription = await Subscription.findOne({
      $or: [{ subscriptionTitle }],
    });
    if (existingSubscription) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.subscription.subscriptionExits
      );
    } else {
      const { error } = validateSubscription(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newSubscriptionData = new Subscription({
          subscriptionTitle,
          planDuration,
          subscriptionAmount,
          startDate,
          endDate,
          status,
          subscriptionContent,
        });
        const savedSubscription = await newSubscriptionData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.subscription.success,
          {
            data: savedSubscription,
          }
        );
      }
    }
  } catch (error) {
    console.log("Getting error while add subscription", error);
    logger.error(`Getting error while add subscription: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing subscription
export const updateSubscription = async (req, res) => {
  const {
    subscriptionTitle,
    planDuration,
    subscriptionAmount,
    startDate,
    endDate,
    status,
    subscriptionContent,
    _id,
  } = req.body;
  const updatedSubscriptionData = {
    subscriptionTitle,
    planDuration,
    subscriptionAmount,
    startDate,
    endDate,
    status,
    subscriptionContent,
  };
  try {
    const { error } = validateSubscription(updatedSubscriptionData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedSubscriptionQuality = await Subscription.findByIdAndUpdate(
        { _id: _id },
        {
          subscriptionTitle,
          planDuration,
          subscriptionAmount,
          startDate,
          endDate,
          status,
          subscriptionContent,
        },
        { new: true }
      );
      if (!updatedSubscriptionQuality) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.subscription.subscriptionNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.subscription.subscriptionUpdated,
        {
          data: updatedSubscriptionQuality,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while update subscription: ${error}`);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all subscriptions
export const getSubscriptions = async (req, res) => {
  try {
    const { search, page, take, role } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [
          { subscriptionTitle: new RegExp(search, "i") },
          { planDuration: new RegExp(search, "i") },
        ],
      };
    }

    let filters = {};
    if (role) filters.role = role;

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };
    const {
      data: subscriptions,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Subscription, options);

    if (!subscriptions.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: subscriptions,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while get subscriptions list : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a subscription
export const deleteSubscription = async (req, res) => {
  const { id } = req.params;
  try {
    await Subscription.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.subscription.subscriptionDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete subscription : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get subscription By Id
export const getSubscriptionById = async (req, res) => {
  try {
    const subscription = await Subscription.findOne({
      _id: req.params.id,
    });
    if (!subscription) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.subscription.subscriptionNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: subscription,
    });
  } catch (error) {
    logger.error(`Getting error while get subscription by id: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
