import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Ticket } from "../entities/ticket.entity.js";
import { validateTicket } from "../validator/validateTicket.js";

// Add a new Ticket
export const addTicket = async (req, res) => {
  const { newLanguage, status } = req.body;

  if (!newLanguage) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingTicket = await Ticket.findOne({
      $or: [{ newLanguage }],
    });
    if (existingTicket) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.ticket.ticketExits
      );
    } else {
      const { error } = validateTicket(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newTicketData = new Ticket({ newLanguage, status });
        const savedTicketData = await newTicketData.save();

        return sendResponse(res, StatusCode.CREATE, messages.ticket.success, {
          data: savedTicketData,
        });
      }
    }
  } catch (error) {
    logger.error(`Getting error while add Ticket : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing Ticket
export const updateTicket = async (req, res) => {
  const { newLanguage, status, _id } = req.body;
  const updatedTicketData = { newLanguage, status };
  try {
    const { error } = validateTicket(updatedTicketData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedTicket = await Ticket.findByIdAndUpdate(
        { _id: _id },
        { newLanguage, status },
        { new: true }
      );
      if (!updatedTicket) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.ticket.ticketNotFound
        );
      }

      return sendResponse(res, StatusCode.OK, messages.ticket.ticketUpdated, {
        data: updatedTicket,
      });
    }
  } catch (error) {
    logger.error(`Getting error while  update language : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all Ticket
export const getTicketes = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ newLanguage: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Ticket, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get ticket : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a Ticket
export const deleteTicket = async (req, res) => {
  const { id } = req.params;
  try {
    await Ticket.findByIdAndDelete({ _id: id });
    return sendResponse(res, StatusCode.OK, messages.ticket.ticketDeleted);
  } catch (error) {
    logger.error(`Getting error while  delete ticket : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get Ticket By Id
export const getTicketById = async (req, res) => {
  try {
    const ticket = await Ticket.findOne({ _id: req.params.id });
    if (!ticket) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.ticket.ticketNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      ticket,
    });
  } catch (error) {
    logger.error(`Getting error while  get ticket by id : ${error}`);

    console.error("Error fetching ticket:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
