import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { TicketSubject } from "../entities/ticketSubject.entity.js";
import { validateTicketSubject } from "../validator/validateTicketSubject.js";

// Add a new TicketSubject
export const addTicketSubject = async (req, res) => {
  const { subjectName, status } = req.body;

  if (!subjectName) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingTicket = await TicketSubject.findOne({
      $or: [{ subjectName }],
    });
    if (existingTicket) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.ticketSubject.ticketSubjectExits
      );
    } else {
      const { error } = validateTicketSubject(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newTicketData = new TicketSubject({ subjectName, status });
        const savedTicketData = await newTicketData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.ticketSubject.success,
          {
            data: savedTicketData,
          }
        );
      }
    }
  } catch (error) {
    logger.error(`Getting error while add TicketSubject : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing TicketSubject
export const updateTicketSubject = async (req, res) => {
  const { subjectName, status, _id } = req.body;
  const updatedTicketData = { subjectName, status };
  try {
    const { error } = validateTicketSubject(updatedTicketData);
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedTicket = await TicketSubject.findByIdAndUpdate(
        { _id: _id },
        { subjectName, status },
        { new: true }
      );
      if (!updatedTicket) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.ticketSubject.ticketSubjectNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.ticketSubject.ticketSubjectUpdated,
        {
          data: updatedTicket,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while  update language : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all TicketSubject
export const getTicketSubjects = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ subjectName: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(TicketSubject, options);
    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while  get ticket : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a TicketSubject
export const deleteTicketSubject = async (req, res) => {
  const { id } = req.params;
  try {
    await TicketSubject.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.ticketSubject.ticketSubjectDeleted
    );
  } catch (error) {
    logger.error(`Getting error while  delete ticket : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get TicketSubject By Id
export const getTicketSubjectById = async (req, res) => {
  try {
    const ticket = await TicketSubject.findOne({ _id: req.params.id });
    if (!ticket) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.ticketSubject.ticketSubjectNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: ticket,
    });
  } catch (error) {
    logger.error(`Getting error while  get ticket subject by id : ${error}`);

    console.error("Error fetching ticket:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
