import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { TranscriptionCategory } from "../entities/transcriptionCategory.entity.js";
import { validateTranscriptionCategory } from "../validator/validateTranscriptionCategory.js";

// Add a new transcription category
export const addTranscriptionCategory = async (req, res) => {
  const { newTranscriptionCategory, status } = req.body;

  if (!newTranscriptionCategory) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingTranscriptionCategory = await TranscriptionCategory.findOne({
      $or: [{ newTranscriptionCategory }],
    });
    if (existingTranscriptionCategory) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.transcriptionCategory.transcriptionCategoryExits
      );
    } else {
      const { error } = validateTranscriptionCategory(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newTranscriptionCategoryData = new TranscriptionCategory({
          newTranscriptionCategory,
          status,
        });
        const savedTranscriptionCategoryData =
          await newTranscriptionCategoryData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.transcriptionCategory.success,
          {
            data: savedTranscriptionCategoryData,
          }
        );
      }
    }
  } catch (error) {
    logger.error(`Getting error while add transcription category: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing transcription category
export const updateTranscriptionCategory = async (req, res) => {
  const { newTranscriptionCategory, status, _id } = req.body;
  const updatedTranscriptionCategorData = { newTranscriptionCategory, status };
  try {
    const { error } = validateTranscriptionCategory(
      updatedTranscriptionCategorData
    );
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedTranscriptionData =
        await TranscriptionCategory.findByIdAndUpdate(
          { _id: _id },
          { newTranscriptionCategory, status },
          { new: true }
        );
      if (!updatedTranscriptionData) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.transcriptionCategory.transcriptionCategoryNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.transcriptionCategory.transcriptionCategoryUpdated,
        {
          data: updatedTranscriptionData,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while update transcription category: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all transcription category
export const getTranscriptionCategoryes = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ newTranscriptionCategory: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(TranscriptionCategory, options);

    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while get transcription categoryes: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a transcription category
export const deleteTranscriptionCategory = async (req, res) => {
  const { id } = req.params;
  try {
    await TranscriptionCategory.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.transcriptionCategory.transcriptionCategoryDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete transcription category: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get transcription category By Id
export const getTranscriptionCategoryById = async (req, res) => {
  try {
    const transcriptionCategory = await TranscriptionCategory.findOne({
      _id: req.params.id,
    });
    if (!transcriptionCategory) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.transcriptionCategory.transcriptionCategoryNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: transcriptionCategory,
    });
  } catch (error) {
    logger.error(
      `Getting error while get transcription category by id: ${error}`
    );

    console.error("Error fetching user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
