import { sendResponse } from "../../../service/common.service.js";
import { logger } from "../../../service/logger.service.js";
import { StatusCode } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { Transcriptionquality } from "../entities/transcriptionquality.entity.js";
import { validateTranscriptionQuality } from "../validator/validateTranscriptionQuality.js";

// Add a new transcriptionQuality
export const addTranscriptionQuality = async (req, res) => {
  const { newTranscriptionQuality, status } = req.body;

  if (!newTranscriptionQuality) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }
  try {
    const existingUser = await Transcriptionquality.findOne({
      $or: [{ newTranscriptionQuality }],
    });
    if (existingUser) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.transcriptionQuality.transcriptionQualityExits
      );
    } else {
      const { error } = validateTranscriptionQuality(req.body);
      if (error) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          error.details.map((detail) => detail.message)
        );
      } else {
        const newTranscriptionQualityData = new Transcriptionquality({
          newTranscriptionQuality,
          status,
        });
        const savedRole = await newTranscriptionQualityData.save();

        return sendResponse(
          res,
          StatusCode.CREATE,
          messages.transcriptionQuality.success,
          {
            data: savedRole,
          }
        );
      }
    }
  } catch (error) {
    logger.error(`Getting error while add transcription quality: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Update an existing transcriptionQuality
export const updateTranscriptionQuality = async (req, res) => {
  const { newTranscriptionQuality, status, _id } = req.body;
  const updatedTranscriptionQualityData = {
    newTranscriptionQuality,
    status,
  };
  try {
    const { error } = validateTranscriptionQuality(
      updatedTranscriptionQualityData
    );
    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    } else {
      const updatedTranscriptionQuality =
        await Transcriptionquality.findByIdAndUpdate(
          { _id: _id },
          { newTranscriptionQuality, status },
          { new: true }
        );
      if (!updatedTranscriptionQuality) {
        return sendResponse(
          res,
          StatusCode.NOT_FOUND,
          messages.transcriptionQuality.transcriptionQualityNotFound
        );
      }

      return sendResponse(
        res,
        StatusCode.OK,
        messages.transcriptionQuality.transcriptionQualityUpdated,
        {
          data: updatedTranscriptionQuality,
        }
      );
    }
  } catch (error) {
    logger.error(`Getting error while update transcription quality: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get all transcriptionQuality
export const getTranscriptionQuality = async (req, res) => {
  try {
    const { search, page, take } = req.query;

    let searchQuery = {};
    if (search) {
      searchQuery = {
        $or: [{ newTranscriptionQuality: new RegExp(search, "i") }],
      };
    }

    let filters = {};

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };

    const {
      data: data,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(Transcriptionquality, options);

    if (!data.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: data,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    logger.error(`Getting error while get transcription qualityes: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Delete a transcriptionQuality
export const deleteTranscriptionQuality = async (req, res) => {
  const { id } = req.params;
  try {
    await Transcriptionquality.findByIdAndDelete({ _id: id });
    return sendResponse(
      res,
      StatusCode.OK,
      messages.transcriptionQuality.transcriptionQualityDeleted
    );
  } catch (error) {
    logger.error(`Getting error while delete transcription quality: ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// Get transcriptionQuality By Id
export const getTranscriptionQualityById = async (req, res) => {
  try {
    const transcriptionQuality = await Transcriptionquality.findOne({
      _id: req.params.id,
    });
    if (!transcriptionQuality) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.transcriptionQuality.transcriptionQualityNotFound
      );
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: transcriptionQuality,
    });
  } catch (error) {
    logger.error(
      `Getting error while get transcription quality by id: ${error}`
    );

    console.error("Error fetching user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
