import {
  getTokenUserInfo,
  sendResponse,
} from "../../../service/common.service.js";
import { sendEmail } from "../../../service/email.service.js";
import { logger } from "../../../service/logger.service.js";
import { generateOtp } from "../../../service/otpService.js";
import { StatusCode, USERTYPE } from "../../../utils/enums.js";
import { messages } from "../../../utils/messages.js";
import { fetchUsersWithFilterAndPagination } from "../../../utils/utils.js";
import { AuthTokenSchema } from "../../auth/entities/authTokenSchema.js";
import { AdminUser, AppUser, User } from "../entities/userSchema.js";
import { validateUser } from "../validateUser.js";
import bcrypt from "bcrypt";

export const addUser = async (req, res) => {
  const { username, phone, email, role, status, password } = req.body;

  // Validate required fields
  if (!username || !email) {
    return sendResponse(res, StatusCode.BADREQUEST, messages.allFieldsRequired);
  }

  try {
    // Get user info from token or default to USER type
    const userInfo = req.headers["authorization"]
      ? getTokenUserInfo(req)
      : { userType: USERTYPE.USER };

    // Build the query for existing users based on email, and optionally phone for ADMINs
    const query = {
      $or: [{ email }],
    };

    if (userInfo.userType === USERTYPE.ADMIN) {
      query.$or.push({ phone });
    }

    // Check if user already exists
    const existingUser = await User.findOne(query);

    // If an existing user is found
    if (existingUser) {
      if (existingUser.userType === USERTYPE.ADMIN) {
        return sendResponse(
          res,
          StatusCode.BADREQUEST,
          messages.signUp.username_emailTaken
        );
      }

      // Handle unverified users (AppUser)
      const existingAppUser = await AppUser.findOne(query);
      if (existingAppUser && !existingAppUser.isVerify) {
        const otp = generateOtp();
        existingUser.otp = otp;
        await existingUser.save();

        let emailHTML = `
          <div class="coupon" style="
            border: 0px;
            width: 95%;
            border-radius: 0px;
            margin: 0 auto;
            max-width: 600px;
            font-family: Arial;
          ">
          <div style="
              padding: 23px 16px;
              background-color: #c4b4fc;
              text-align: center;
              border-radius: 0 0;
            ">
            <a href="#">
              <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                style="padding: 10px 0px;max-width: 200px; height: auto;" />
            </a>
          </div>
          <div style="background-color: white; padding: 0 16px">
            <p style="
                font-size: 14px;
                font-family: Arial, sans-serif;
                color: #2b303a;
                margin: 25px 0 0;
                text-transform: capitalize;
              ">
              Dear <b>${existingUser.username}</b>
            </p>
          </div>
          <div style="font-family: sans-serif; padding: 10px 16px 50px">
            <div class="" style="
                font-size: 12px;
                font-family: Arial, sans-serif;
                mso-line-height-alt: 18px;
                color: #555555;
                line-height: 1.5;
              ">
              <p style="
                  margin: 0;
                  font-size: 14px;
                  text-align: left;
                  mso-line-height-alt: 22.5px;
                "></p>
              <p>Please use this code to verify your email.</p>
              <p style="
                  font-size: 16px;">
                <b>OTP : </b> ${otp}
              </p>
              <p>
                <span>Best Regards,</span><br /><span><b>Summary AI Team</b></span>
              </p>
            </div>
          </div>

          <div style="background-color: #c4b4fc">
            <div style="
                padding: 2px 16px;
                text-align: center;
                border-radius: 0 0;
                border: 0;
              ">
              <a href="#" style="border: 0">
                <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                  style="padding: 10px 0px;max-width: 200px; height: auto;" />
              </a>
            </div>
            <div align="center" class="alignment" style="padding: 2px 16px; text-align: center; border-radius: 0 0">
              <div style="padding: 2px 16px; text-align: center; border-radius: 0 0">
                <div class="" style="
                    font-size: 14px;
                    font-family: Arial, sans-serif;
                    mso-line-height-alt: 14.399999999999999px;
                    color: #555555;
                    line-height: 1.2;
                  ">
                  <p style="
                      margin: 0;
                      font-size: 14px;
                      text-align: center;
                      mso-line-height-alt: 16.8px;
                      padding: 0 0 18px 0;
                    ">
                    <span style="color: #000; font-size: 14px">Copyright © 2024 Summary AI Services - All rights
                      reserved.</span>
                  </p>
                </div>
              </div>
            </div>
          </div>
        </div>`;

        const emailSent = await sendEmail({
          from: process.env.SENDER_EMAIL,
          to: existingUser.email,
          subject: "Summary AI Verification OTP",
          html: emailHTML,
        });

        if (!emailSent) {
          return sendResponse(
            res,
            StatusCode.INTERNAL_SERVER_ERROR,
            "Error sending verification email"
          );
        }

        return sendResponse(res, StatusCode.OK, messages.signIn.otpSent, {
          otp,
          ...existingAppUser._doc,
        });
      }

      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        messages.signUp.username_emailTaken
      );
    }

    // Validate the new user's data
    const { error } = validateUser({
      ...req.body,
      userType: userInfo.userType,
    });

    if (error) {
      return sendResponse(
        res,
        StatusCode.BADREQUEST,
        error.details.map((detail) => detail.message)
      );
    }

    // Hash password if not an ADMIN
    let hashedPassword;
    if (userInfo.userType !== USERTYPE.ADMIN) {
      hashedPassword = await bcrypt.hash(password, 10);
    }

    // Create the new user
    const newUser = new User({
      username,
      email,
      password: hashedPassword,
      role,
      userType: userInfo.userType,
      status: userInfo.status,
    });
    const savedUser = await newUser.save();
    await savedUser.populate("role", "roleName _id");

    // Save specific user type (AdminUser or AppUser)
    if (userInfo.userType === USERTYPE.ADMIN) {
      const adminUser = new AdminUser({
        userId: savedUser._id,
        username,
        phone,
        email,
        profileImage: req.file?.filename,
        role,
        userType: userInfo.userType,
        status,
      });
      await adminUser.save();
    } else {
      const appUser = new AppUser({
        userId: savedUser._id,
        username,
        email,
        phone,
        userType: userInfo.userType,
      });
      await appUser.save();

      // Send OTP for email verification
      const otp = generateOtp();
      newUser.otp = otp;
      await newUser.save();
      let emailHTML = `
          <div class="coupon" style="
            border: 0px;
            width: 95%;
            border-radius: 0px;
            margin: 0 auto;
            max-width: 600px;
            font-family: Arial;
          ">
          <div style="
              padding: 23px 16px;
              background-color: #c4b4fc;
              text-align: center;
              border-radius: 0 0;
            ">
            <a href="#">
              <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                style="padding: 10px 0px;max-width: 200px; height: auto;" />
            </a>
          </div>
          <div style="background-color: white; padding: 0 16px">
            <p style="
                font-size: 14px;
                font-family: Arial, sans-serif;
                color: #2b303a;
                margin: 25px 0 0;
                text-transform: capitalize;
              ">
              Dear <b>${appUser.username}</b>
            </p>
          </div>
          <div style="font-family: sans-serif; padding: 10px 16px 50px">
            <div class="" style="
                font-size: 12px;
                font-family: Arial, sans-serif;
                mso-line-height-alt: 18px;
                color: #555555;
                line-height: 1.5;
              ">
              <p style="
                  margin: 0;
                  font-size: 14px;
                  text-align: left;
                  mso-line-height-alt: 22.5px;
                "></p>
              <p>Please use this code to verify your email.</p>
              <p style="
                  font-size: 16px;">
                <b>OTP : </b> ${otp}
              </p>
              <p>
                <span>Best Regards,</span><br /><span><b>Summary AI Team</b></span>
              </p>
            </div>
          </div>

          <div style="background-color: #c4b4fc">
            <div style="
                padding: 2px 16px;
                text-align: center;
                border-radius: 0 0;
                border: 0;
              ">
              <a href="#" style="border: 0">
                <img src="http://148.72.246.221:81/summary-ai/app/_next/static/media/logo.db9ce2e5.png" alt="logo"
                  style="padding: 10px 0px;max-width: 200px; height: auto;" />
              </a>
            </div>
            <div align="center" class="alignment" style="padding: 2px 16px; text-align: center; border-radius: 0 0">
              <div style="padding: 2px 16px; text-align: center; border-radius: 0 0">
                <div class="" style="
                    font-size: 14px;
                    font-family: Arial, sans-serif;
                    mso-line-height-alt: 14.399999999999999px;
                    color: #555555;
                    line-height: 1.2;
                  ">
                  <p style="
                      margin: 0;
                      font-size: 14px;
                      text-align: center;
                      mso-line-height-alt: 16.8px;
                      padding: 0 0 18px 0;
                    ">
                    <span style="color: #000; font-size: 14px">Copyright © 2024 Summary AI Services - All rights
                      reserved.</span>
                  </p>
                </div>
              </div>
            </div>
          </div>
        </div>`;

      const emailSent = await sendEmail({
        from: process.env.SENDER_EMAIL,
        to: newUser.email,
        subject: "Summary AI Verification OTP",
        html: emailHTML,
      });

      if (!emailSent) {
        return sendResponse(
          res,
          StatusCode.INTERNAL_SERVER_ERROR,
          "Error sending verification email"
        );
      }

      return sendResponse(res, StatusCode.OK, messages.signIn.otpSent, {
        otp,
        ...appUser._doc,
      });
    }

    return sendResponse(res, StatusCode.CREATE, messages.user.success, {
      savedUser,
    });
  } catch (error) {
    logger.error(`Getting error while add-user: ${error}`);
    console.error("error", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getUsers = async (req, res) => {
  try {
    const { search, page, take, role } = req.query;
    let userInfo = getTokenUserInfo(req);
    let searchQuery = {
      username: {
        $nin: ["Super Admin", userInfo?.username],
      },
    };
    if (search) {
      searchQuery = {
        $and: [
          { username: { $nin: ["Super Admin", userInfo?.username] } },
          { username: new RegExp(search, "i") },
        ],
      };
    }
    let filters = {};
    if (role) filters.role = role;

    const options = {
      searchQuery,
      filter: filters,
      page: parseInt(page) || 1,
      take: parseInt(take) || 10,
    };
    const {
      data: users,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(AdminUser, options);

    const populatedUsers = await AdminUser.populate(users, {
      path: "role",
      select: "roleName",
    });

    if (!users.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }
    return sendResponse(res, StatusCode.OK, messages.success, {
      data: populatedUsers,
      pagination: {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      },
    });
  } catch (error) {
    console.error("Error fetching users:", error);
    logger.error(`Getting error while fetching users : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const getUser = async (req, res) => {
  try {
    let user = await AdminUser.findOne({ _id: req.params.id }).populate(
      "role",
      "roleName _id"
    );

    if (!user) {
      user = await AppUser.findOne({ _id: req.params.id });
    }

    if (!user) {
      return sendResponse(res, StatusCode.OK, messages.noUsersFound);
    }

    return sendResponse(res, StatusCode.OK, messages.success, { data: user });
  } catch (error) {
    console.error("Error fetching user:", error);
    logger.error(`Getting error while fetching user : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// export const getUser = async (req, res) => {
//   try {
//     const adminUser = await AdminUser.findOne({ _id: req.params.id }).populate(
//       "role",
//       "roleName _id"
//     );

//     if (!adminUser) {
//       return sendResponse(res, StatusCode.OK, messages.noUsersFound);
//     }
//     return sendResponse(res, StatusCode.OK, messages.success, {
//        adminUser,
//     });
//   } catch (error) {
//     console.error("Error fetching user:", error);
//     logger.error(`Getting error while fetching user : ${error}`);

//     return sendResponse(
//       res,
//       StatusCode.INTERNAL_SERVER_ERROR,
//       messages.internalServerError
//     );
//   }
// };

// export const getAppUsers = async (req, res) => {
//   try {
//     const data = await AppUser.find();
//     if (!data.length) {
//       return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
//     }
//     return sendResponse(res, StatusCode.OK, messages.success, {
//
//          data,
//
//     });
//   } catch (error) {
//     return sendResponse(
//       res,
//       StatusCode.INTERNAL_SERVER_ERROR,
//       messages.internalServerError
//     );
//   }
// };

// export const deleteUser = async (req, res) => {
//   try {
//     const userInfo = getTokenUserInfo(req);
//     const userId = req.params.id;
//     let user, mainUserId;

//     if (userInfo.userType === USERTYPE.ADMIN) {
//       const adminUser = await AdminUser.findOneAndDelete({ _id: userId });
//       if (!adminUser) {
//         return sendResponse(res, StatusCode.NOT_FOUND, messages.noUsersFound);
//       }
//       mainUserId = adminUser.userId;
//     } else {
//       const appUser = await AppUser.findOneAndDelete({ _id: userId });
//       if (!appUser) {
//         return sendResponse(res, StatusCode.NOT_FOUND, messages.noUsersFound);
//       }
//       mainUserId = appUser.userId;
//     }

//     user = await User.findOneAndDelete({ _id: mainUserId });
//     if (!user) {
//       return sendResponse(res, StatusCode.NOT_FOUND, messages.noUsersFound);
//     }

//     return sendResponse(res, StatusCode.OK, messages.delete);
//   } catch (error) {
//     console.error("Error deleting user:", error);
//     logger.error(`Getting error while deleting user : ${error}`);

//     return sendResponse(
//       res,
//       StatusCode.INTERNAL_SERVER_ERROR,
//       messages.internalServerError
//     );
//   }
// };

export const deleteUser = async (req, res) => {
  try {
    let userInfo = getTokenUserInfo(req);
    console.log("userInfo", userInfo?.userType == USERTYPE.USER);
    const userId = req.params.id;
    let user;
    if (userInfo?.userType == USERTYPE.USER) {
      user = await AppUser.findOneAndDelete({ _id: userId });
    } else {
      user = await AdminUser.findOneAndDelete({ _id: userId });
    }
    if (!user) {
      return sendResponse(res, StatusCode.NOT_FOUND, messages.noUsersFound);
    }
    const mainUserId = user.userId;
    await User.findOneAndDelete({ _id: mainUserId }).then(async (res) => {
      await AuthTokenSchema.deleteMany({ userId: mainUserId });
    });
    return sendResponse(res, StatusCode.OK, messages.delete);
  } catch (error) {
    console.error("Error deleting user:", error);
    logger.error(`Getting error while deleting user : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

export const updateUser = async (req, res) => {
  const { username, email, password, _id, role, status, phone } = req.body;
  const profileImage = req.file ? req.file.filename : null;
  let userInfo;

  try {
    userInfo = await AdminUser.findOne({ _id: _id });
    if (!userInfo) {
      userInfo = await AppUser.findOne({ _id: _id });
    }
    const updateData = {
      username,
      email,
      password,
      role,
      status,
      userType: userInfo.userType,
      phone,
    };
    if (profileImage) {
      updateData.profileImage = profileImage;
    }

    // Validate the update data
    // const { error } = validateUser(updateData);
    // if (error) {
    //   return sendResponse(
    //     res,
    //     StatusCode.BADREQUEST,
    //     error.details.map((detail) => detail.message)
    //   );
    // }

    let updatedUser;
    let mainUserId;

    // Attempt to update the user in AdminUser collection
    updatedUser = await AdminUser.findOneAndUpdate(
      { _id: _id },
      { $set: updateData },
      { new: true }
    );

    if (!updatedUser) {
      // If user is not found in AdminUser, try updating in AppUser collection
      updatedUser = await AppUser.findOneAndUpdate(
        { _id: _id },
        { $set: updateData },
        { new: true }
      );
    }

    if (!updatedUser) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.updateUser.userNotFound
      );
    }

    mainUserId = updatedUser.userId;

    // Update the user in the User collection
    const updatedMainUser = await User.findOneAndUpdate(
      { _id: mainUserId },
      { $set: updateData },
      { new: true }
    );

    if (!updatedMainUser) {
      return sendResponse(
        res,
        StatusCode.NOT_FOUND,
        messages.updateUser.userNotFound
      );
    }

    return sendResponse(res, StatusCode.OK, messages.updateUser.success, {
      ...updatedUser._doc,
      ...updatedMainUser._doc,
    });
  } catch (error) {
    logger.error(`Getting error while updating user : ${error}`);

    console.error("Error updating user:", error);
    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};

// export const updateUser = async (req, res) => {
//   const { username, email, password, _id, role, status, phone } = req.body;
//   const profileImage = req.file ? req.file.filename : null;

//   try {
//     let userInfo = getTokenUserInfo(req);

//     const updateData = {
//       username,
//       email,
//       password,
//       role,
//       status,
//       userType: userInfo.userType,
//       phone,
//     };
//     if (profileImage) {
//       updateData.profileImage = profileImage;
//     }

//     // Validate the update data
//     const { error } = validateUser(updateData);
//     if (error) {
//       return sendResponse(
//         res,
//         StatusCode.BADREQUEST,
//         error.details.map((detail) => detail.message)
//       );
//     }

//     let updatedUser;
//     let mainUserId;

//     if (userInfo.userType === USERTYPE.ADMIN) {
//       updatedUser = await AdminUser.findOneAndUpdate(
//         { _id: _id },
//         { $set: updateData },
//         { new: true }
//       );
//       if (updatedUser) {
//         mainUserId = updatedUser.userId;
//       }
//     } else {
//       updatedUser = await AppUser.findOneAndUpdate(
//         { _id: _id },
//         { $set: updateData },
//         { new: true }
//       );
//       if (updatedUser) {
//         mainUserId = updatedUser.userId;
//       }
//     }
//     if (!updatedUser) {
//       return sendResponse(
//         res,
//         StatusCode.NOT_FOUND,
//         messages.updateUser.userNotFound
//       );
//     }

//     // Update the user in the User collection
//     const updatedMainUser = await User.findOneAndUpdate(
//       { _id: mainUserId },
//       { $set: updateData },
//       { new: true }
//     );

//     if (!updatedMainUser) {
//       return sendResponse(
//         res,
//         StatusCode.NOT_FOUND,
//         messages.updateUser.userNotFound
//       );
//     }

//     return sendResponse(res, StatusCode.OK, messages.updateUser.success, {
//       data: { ...updatedUser._doc, ...updatedMainUser._doc },
//     });
//   } catch (error) {
//     logger.error(`Getting error while updating user : ${error}`);

//     console.error("Error updating user:", error);
//     return sendResponse(
//       res,
//       StatusCode.INTERNAL_SERVER_ERROR,
//       messages.internalServerError
//     );
//   }
// };

export const getAppUsers = async (req, res) => {
  try {
    const { search, page, take, role, isDropDown } = req.query;
    let userInfo = getTokenUserInfo(req);

    let searchQuery = { _id: { $ne: userInfo._id } };
    if (search) {
      searchQuery = {
        $or: [
          { username: new RegExp(search, "i") },
          { email: new RegExp(search, "i") },
        ],
      };
    }

    let filters = {};
    if (role) filters.role = role;

    let options;
    if (isDropDown === "true") {
      options = {
        searchQuery,
        filter: filters,
      };
    } else {
      options = {
        searchQuery,
        filter: filters,
        page: parseInt(page) || 1,
        take: parseInt(take) || 10,
      };
    }

    const {
      data: users,
      totalData,
      page: currentPage,
      take: perPage,
      totalPages,
      prevPage,
      nextPage,
    } = await fetchUsersWithFilterAndPagination(AppUser, options);

    const populatedUsers = await AppUser.populate(users, {
      path: "role",
      select: "roleName",
    });

    if (!users.length) {
      return sendResponse(res, StatusCode.OK, messages.noDataAvailable);
    }

    const responsePayload = {
      data: populatedUsers,
    };

    if (isDropDown !== "true") {
      responsePayload.pagination = {
        nextPage,
        page: currentPage,
        prevPage,
        take: perPage,
        totalData,
        totalPages,
      };
    }

    return sendResponse(res, StatusCode.OK, messages.success, {
      data: responsePayload,
    });
  } catch (error) {
    console.error("Error fetching users:", error);
    logger.error(`Getting error while fetching users : ${error}`);

    return sendResponse(
      res,
      StatusCode.INTERNAL_SERVER_ERROR,
      messages.internalServerError
    );
  }
};
